<?php
/**
 * This file handles Adyen Common Action

 * @package WK Adyen Checkout Api
 *
 * @version 1.0.0
 */

namespace WkwcAdyen\Helper\Common;

defined( 'ABSPATH' ) || exit; // Exit if access directly.

use WkwcAdyen\Helper\Adyen\Wkwc_Adyen_Helper;

if ( ! class_exists( 'Wkwc_Adyen_Checkout_Action' ) ) {

	/**
	 * Users class.
	 */
	class Wkwc_Adyen_Checkout_Action {
		/**
		 * $adyen_helper
		 *
		 * @var Wkwc_Adyen_Helper
		 */
		public $adyen_helper;

		/**
		 * Checkout base url
		 *
		 * @var $base_url
		 */
		public $base_url;

		/**
		 * $api_handler
		 *
		 * @var Wkwc_Adyen_Api_Handler
		 */
		public $api_handler;

		/**
		 * Adyen Config Instance
		 *
		 * @var Wkwc_Adyen_Config
		 */
		public $adyen_config;

		/**
		 * Version of the API.
		 *
		 * @var string
		 */
		protected $version = 'v71';

		/**
		 * Adyen_Checkout_Action constructor.
		 */
		public function __construct() {
			$this->adyen_helper = new Wkwc_Adyen_Helper();
			$this->adyen_config = $this->adyen_helper->adyen_config;
			$this->base_url     = $this->wkwc_adyen_checkout_base_url();

			$this->api_handler = new Wkwc_Adyen_Api_Handler( $this->base_url );
		}



		/**
		 * Get checkout base url.
		 *
		 * @return string
		 */
		public function wkwc_adyen_checkout_base_url() {

			$url = 'https://' . $this->adyen_config->wkwc_adyen_get_live_endpoint_prefix() . '-checkout-live.' . $this->adyen_helper->wkwc_adyen_get_live_domain() . '/checkout/' . $this->version;

			if ( $this->adyen_config->wkwc_adyen_is_test_mode() ) {
				$url = 'https://checkout-test.' . $this->adyen_helper->wkwc_adyen_get_test_domain() . '/' . $this->version;
			}

			return $url;
		}

		/**
		 * Retrieves the entire list of payment methods.
		 *
		 * @param string $country country code.
		 * @param float  $amount amount to be charged.
		 * @param bool   $cached - whether or not to use cached data.
		 * @return array
		 */
		public function wkwc_adyen_list_payment_methods( $country = null, float $amount = 0, $cached = true ) {

			$result = array();

			if ( $cached ) {
				$result = get_transient( WKWC_ADN_PREFIX . '_payment_methods_' . $country );
			}

			if ( empty( $result ) ) {
				$payload = array(
					'merchantAccount'  => $this->adyen_config->wkwc_adyen_get_merchantcode(),
					'shopperReference' => $this->adyen_helper::wkwc_adyen_get_shopper_reference(),
					'channel'          => 'Web',
				);

				if ( ! is_null( $country ) ) {
					$payload = array_merge(
						$payload,
						array(
							'countryCode' => $country,
							'amount'      => array(
								'currency' => get_woocommerce_currency(),
								'value'    => $this->wkwc_adyen_format_amount( $amount ),
							),
						)
					);
				}

				$response = $this->api_handler->wkwc_adyen_remote_post( 'paymentMethods', $payload );

				if ( 200 === $response->status ) {
					$result = $this->adyen_helper::wkwc_adyen_obj_to_arr( $response->body );

					set_transient( WKWC_ADN_PREFIX . '_payment_methods_' . $country, $result, \DAY_IN_SECONDS );
				}
			}

			return $result;
		}

		/**
		 * Retrieves the available stored payment methods.
		 *
		 * @param string $country country code.
		 * @param float  $amount amount to be charged.
		 * @return array
		 */
		public function wkwc_adyen_get_stored_payment_methods( $country = null, float $amount = 0 ) {
			$reference = $this->adyen_helper::wkwc_adyen_get_shopper_reference();
			$result    = array_filter( (array) get_transient( 'wkwc_adyen_stored_payment_methods_' . $reference ) );

			if ( empty( $result ) && is_user_logged_in() ) {
				$response = $this->wkwc_adyen_list_payment_methods( $country, $amount, false );

				if ( isset( $response['storedPaymentMethods'] ) ) {
					$result = $response['storedPaymentMethods'];

					set_transient( 'wkwc_adyen_stored_payment_methods_' . $reference, $result, DAY_IN_SECONDS );
				}
			}

			return $result;
		}

		/**
		 * Retrieves the available payment methods.
		 *
		 * @param string $country country code.
		 * @param float  $amount amount to be charged.
		 * @return array
		 */
		public function wkwc_adyen_get_payment_methods( $country = null, float $amount = 0 ) {
			$result   = array();
			$response = $this->wkwc_adyen_list_payment_methods( $country, $amount );

			if ( isset( $response['paymentMethods'] ) ) {
				$result = $response['paymentMethods'];
			}

			return $result;
		}


		/**
		 * Retrieves the available card types.
		 *
		 * @param string $country country code.
		 * @return array
		 */
		public function wkwc_adyen_get_card_types( $country = null ) {
			$result   = array();
			$response = $this->wkwc_adyen_list_payment_methods( $country );

			if ( isset( $response['paymentMethods'] ) ) {
				foreach ( $response['paymentMethods'] as $item ) {
					if ( 'scheme' === $item['type'] ) {
						$result = $item['brands'];
					}
				}
			}

			return $result;
		}

		/**
		 * Retrieves the stored cards which have `Ecommerce` support.
		 *
		 * @param string $country country code.
		 * @param float  $amount amount to be charged.
		 * @return array
		 */
		public function wkwc_adyen_get_ec_stored_cards( $country = null, float $amount = 0 ) {

			$result = array();
			$list   = ! empty( $this->wkwc_adyen_get_stored_payment_methods( $country, $amount ) ) ? $this->wkwc_adyen_get_stored_payment_methods( $country, $amount ) : array();

			foreach ( $list as $item ) {
				if ( isset( $item['lastFour'] ) && in_array( 'Ecommerce', $item['supportedShopperInteractions'], true ) ) {
					$result[] = $item;
				}
			}

			return $result;
		}

		/**
		 * Sends a payment.
		 *
		 * @param array $payload request payload.
		 * @return object
		 */
		public function wkwc_adyen_send_payment( $payload ) {

			$payload = array_merge( $payload, $this->adyen_helper->wkwc_adyen_app_info(), $this->adyen_helper->wkwc_adyen_capture_immediately() ); // add our app info in the payload.

			$response = $this->api_handler->wkwc_adyen_remote_post( 'payments', $payload );

			return $response;
		}

		/**
		 * Sends the payment details.
		 *
		 * @param array $payload request payload.
		 * @return object
		 */
		public function wkwc_adyen_send_payment_details( $payload ) {

			$response = $this->api_handler->wkwc_adyen_remote_post( 'payments/details', $payload );

			return $response;
		}



		/**
		 * Refunds a payment.
		 *
		 * @param string $reference - payment reference to be refunded.
		 * @param float  $amount - amount to be refunded.
		 * @return object
		 */
		public function wkwc_adyen_refund_payment( $reference, $amount ) {

			$payload = array(
				'amount'          => array(
					'currency' => get_woocommerce_currency(),
					'value'    => $this->adyen_helper->wkwc_adyen_format_amount( $amount ),
				),
				'merchantAccount' => $this->adyen_config->wkwc_adyen_get_merchantcode(),
				'reference'       => $reference,
			);

			$response = $this->api_handler->wkwc_adyen_remote_post( "payments/{$reference}/refunds", $payload );

			return $response;
		}


		/**
		 * Cancels the payment.
		 *
		 * @param string $reference - payment reference to be canceled.
		 * @return object
		 */
		public function wkwc_adyen_cancel_payment( $reference ) {

			$payload = array(
				'merchantAccount' => $this->adyen_config->wkwc_adyen_get_merchantcode(),
				'reference'       => $reference,
			);

			$response = $this->api_handler->wkwc_adyen_remote_post( "payments/{$reference}/cancels", $payload );

			return $response;
		}



		/**
		 * Captures an authorised payment.
		 *
		 * @param string $reference - payment reference to be captured.
		 * @param float  $amount - amount to be captured.
		 * @return object
		 */
		public function wkwc_adyen_capture_payment( $reference, $amount ) {

			$payload = array(
				'amount'          => array(
					'currency' => get_woocommerce_currency(),
					'value'    => $this->adyen_helper->wkwc_adyen_format_amount( $amount ),
				),
				'merchantAccount' => $this->adyen_config->wkwc_adyen_get_merchantcode(),
				'reference'       => $reference,
			);

			$response = $this->api_handler->wkwc_adyen_remote_post( "payments/{$reference}/captures", $payload );

			return $response;
		}

		/**
		 * Get payment links details by id.
		 *
		 * @param string $link_id - payment link id.
		 * @return object
		 */
		public function wkwc_adyen_payment_link_details( $link_id ) {

			$response = $this->api_handler->wkwc_adyen_remote_get( "paymentLinks/{$link_id}" );

			return $response;
		}
	}
}
