<?php
/**
 * Google Pay
 *
 * @package WkwcAdyen\Includes\Payment_Methods
 * @version 1.0.0
 */

namespace WkwcAdyen\Includes\Payment_Methods;

// prevent direct access data leaks.
use Automattic\WooCommerce\StoreApi\Utilities\NoticeHandler;

defined( 'ABSPATH' ) || exit;

use WkwcAdyen\Helper\Common\Wkwc_Adyen_Service_Util;

use WkwcAdyen\Helper\Common\Wkwc_Adyen_Checkout_Action;

/**
 * Wkwc_Adyen_Applepay class.
 */
class Wkwc_Adyen_Applepay extends Wkwc_Adyen_Abstract_Gateway {
	/**
	 * Checkout action instance
	 *
	 * @var Wkwc_Adyen_Checkout_Action
	 */
	public $checkout_action;

	/**
	 * Constructor of this class.
	 *
	 * @param bool $init_hooks Whether or not to initialize hooks.
	 * @since 1.0.4
	 * @since 1.1.0 - add support for subscriptions
	 */
	public function __construct( $init_hooks = true ) {
		parent::__construct( $init_hooks );
		$this->checkout_action = new Wkwc_Adyen_Checkout_Action();
		$this->has_fields      = true;
	}

	/**
	 * Gets default payment method title.
	 *
	 * @return string
	 */
	public function get_default_title() {
		return __( 'Adyen - Apple Pay', 'wkwc-adyen' );
	}

	/**
	 * Gets default payment method description.
	 *
	 * @since 1.1.0 - display supported countries
	 * @since 1.0.4
	 * @return string
	 */
	public function get_default_description() {
		return $this->show_supported_country();
	}

	/**
	 * Gets default description set in settings.
	 *
	 * @return void|string The default description
	 */
	public function get_settings_description() {
		$this->description = ( $this->settings['description'] ) ? $this->settings['description'] : __( 'Pay with apple pay', 'wkwc-adyen' );
		return $this->description;
	}

	/**
	 * Type of the payment method (e.g ideal, scheme. bcmc).
	 *
	 * @return string
	 */
	public function payment_method_type() {
		return 'applepay';
	}

	/**
	 * Returns the payment method to be used for recurring payments
	 *
	 * @return void
	 */
	public function recurring_payment_method() {
		// Do nothing.
	}

	/**
	 * Adds an array of fields to be displayed on the gateway's settings screen.
	 *
	 * @since 1.1.0
	 * @return void
	 */
	public function init_form_fields() {

		$is_enabled = parent::init_form_fields();

		if ( false === $is_enabled ) {
			return;
		}

		if ( 'applepay' === $this->payment_method_type() ) {
			$this->form_fields = array_merge(
				$this->form_fields,
				array(
					'apple_token_error'           => array(
						'title'       => __( 'Token Error', 'wkwc-adyen' ),
						'type'        => 'text',
						'description' => __( 'Provide a token error message to display as an error for the Apple token', 'wkwc-adyen' ),
					),
					'no_apple_device_description' => array(
						'title'       => __( 'Description for Non-Apple Devices', 'wkwc-adyen' ),
						'type'        => 'text',
						'description' => __( 'Provide a description message to display for non-Apple devices.', 'wkwc-adyen' ),
						'default'     => __( 'Please choose an apple device', 'wkwc-adyen' ),
					),
					'config_desc'                 => array(
						'title' => '',
						'label' => '',
						'type'  => 'config_desc',
					),
				)
			);
		}
	}

	/**
	 * Generates the HTML for `config_desc` field type
	 *
	 * @since 1.1.0
	 * @return string
	 */
	public function generate_config_desc_html() {
		ob_start();
		?>
		<tr valign="top">
		<td colspan="2" class="forminp" style="padding: 0;">
			<h3><?php esc_html_e( 'Configuration', 'wkwc-adyen' ); ?></h3>
			<ol>
				<li>
					<p>
						<?php
						printf(
						/* translators: %1$s: link to Apple Pay certificate download, %2$s: closing tag. */
							esc_html__( 'Download and unzip the %1$sAdyen\'s Apple Pay certificate%2$s', 'wkwc-adyen' ),
							'<a href="' . esc_url( 'https://docs.adyen.com/payment-methods/apple-pay/web-component/apple-developer-merchantid-domain-association-2024.zip' ) . '" target="_blank">',
							'</a>'
						);
						?>
					</p>
				</li>
				<li>
					<p>
					<?php
					printf(
						/* translators: %1$s: bold tag, %2$s: closing tag, %3$s: code tag. */
						esc_html__( 'Host the certificate file with the name %1$sapple-developer-merchantid-domain-association%2$s on each domain you want to use, including subdomains, under the following path: %3$s', 'wkwc-adyen' ),
						'<b>',
						'</b>',
						'<code>/.well-known/apple-developer-merchantid-domain-association</code>'
					);
					?>
						</p>
					<p><?php esc_html_e( 'The file must:', 'wkwc-adyen' ); ?></p>
					<ul style="list-style: disc;padding-left: 20px;">
					<li><?php esc_html_e( 'Be externally accessible.', 'wkwc-adyen' ); ?></li>
					<li><?php esc_html_e( 'Not be password protected.', 'wkwc-adyen' ); ?></li>
					<li><?php esc_html_e( 'The file cannot be behind a proxy or redirect.', 'wkwc-adyen' ); ?></li>
					</ul>
					<p>
					<?php
					printf(
						/* translators: %1$s: link to Apple Pay certificate upload, %2$s: closing tag. */
						esc_html__( 'See an example of a %1$sworking certificate file.%2$s', 'wkwc-adyen' ),
						'<a href="' . esc_url( 'https://eu.adyen.link/.well-known/apple-developer-merchantid-domain-association' ) . '" target="_blank">',
						'</a>'
					);
					?>
						</p>
				</li>
				<li>
					<p>
					<?php
					printf(
						/* translators: %1$s: link to Apple Pay add payment methods, %2$s: closing tag. */
						esc_html__( '%1$sAdd Apple Pay in your live Customer Area%2$s, where you will be asked for:', 'wkwc-adyen' ),
						'<a href="' . esc_url( 'https://docs.adyen.com/payment-methods/add-payment-methods/' ) . '" target="_blank">',
						'</a>'
					);
					?>
						</p>
					<ul style="list-style: disc;padding-left: 20px;">
					<li><b>Shop website:</b>
					<?php
					printf(
						/* translators: %s: code tag. */
						esc_html__( 'your main website URL, for example %s.', 'wkwc-adyen' ),
						'<code>https://www.mystore.com</code>'
					);
					?>
						</li>
					<li><b>Additional shop websites:</b>
					<?php
					printf(
						/* translators: %s: code tag. */
						esc_html__( 'Add any other domains you use for your shop website, for example %1$s, or %2$s. You must register all top-level domains and subdomains.', 'wkwc-adyen' ),
						'<code>https://www.mystore1.com</code>',
						'<code>https://www.mystore1.com</code>'
					);
					?>
						</li>
					</ul>
				</li>
			</ol>
		</td>
		</tr>
		<?php
		return ob_get_clean();
	}

	/**
	 * Adds extra fields.
	 *
	 * @return void
	 */
	public function payment_fields() {

		parent::payment_fields();

		echo $this->wkwc_adyen_generate_extra_applepay_fields_html(); // phpcs:ignore
	}

	/**
	 * Generates extra fields HTML.
	 *
	 * @return void
	 */
	public function wkwc_adyen_generate_extra_applepay_fields_html() {
		?>
		<div id="wkwc-adyen-applepay-container"></div>
		<input type="hidden" id="<?php echo esc_attr( $this->id . '_token' ); ?>" name="<?php echo esc_attr( $this->id . '_token' ); ?>">
		<input type="hidden" id="<?php echo esc_attr( $this->id . '_nonce' ); ?>" name="<?php echo esc_attr( $this->id . '_nonce' ); ?>" value='<?php echo esc_attr( wp_create_nonce( $this->id . '_nonce' ) ); ?>'>
		<?php
	}


	/**
	 * Validates extra added fields.
	 *
	 * @return bool
	 */
	public function validate_fields() {
		$is_valid = parent::validate_fields();
		// Verify the nonce for security.
		if ( ! \WC_Blocks_Utils::has_block_in_page( wc_get_page_id( 'checkout' ), 'woocommerce/checkout' ) ) {
			$nonce = isset( $_POST[ $this->id . '_nonce' ] ) ? sanitize_text_field( $_POST[ $this->id . '_nonce' ] ) : '';
			if ( ! wp_verify_nonce( $nonce, $this->id . '_nonce' ) ) {
				wp_die( esc_html__( 'Nope! Security check failed!', 'wkwc-adyen' ) );
				$is_valid = false;
			}
		}

		wkwc_adn_log( 'Apple Pay Request data: ' . wp_json_encode( $_POST ) ); // phpcs:ignore

		// Check if the token exists and is not empty.
		if ( isset( $_POST[ $this->id . '_token' ] ) ) {
			$token = sanitize_text_field( $_POST[ $this->id . '_token' ] );

			if ( empty( $token ) ) {
				$apple_token_error = $this->get_option( 'apple_token_error' );
				wc_add_notice( $apple_token_error, 'error' );
				$is_valid = false;
			}
		} else {
			// If the token field is not set, return an error.
			$apple_token_error = $this->get_option( 'apple_token_error' );
			wc_add_notice( $apple_token_error, 'error' );
			$is_valid = false;
		}

		return $is_valid;
	}


	/**
	 * Builds the required payment payload
	 *
	 * @param \WC_Order $order Order object.
	 * @param string    $reference Payment reference.
	 * @return array
	 */
	public function wkwc_adyen_build_applepay_payment_payload( \WC_Order $order, $reference ) {

		// Get the token from the POST data.
		// Verify the nonce for security.
		if ( ! \WC_Blocks_Utils::has_block_in_page( wc_get_page_id( 'checkout' ), 'woocommerce/checkout' ) ) {
			$nonce = isset( $_POST[ $this->id . '_nonce' ] ) ? sanitize_text_field( $_POST[ $this->id . '_nonce' ] ) : '';
			if ( ! wp_verify_nonce( $nonce, $this->id . '_nonce' ) ) {
				wp_die( esc_html__( 'Nope! Security check failed!', 'wkwc-adyen' ) );
			}
		}

		$token = '';
		if ( isset( $_POST[ $this->id . '_token' ] ) ) {
			$token = sanitize_text_field( $_POST[ $this->id . '_token' ] );
		}

		$payload = array_merge(
			parent::wkwc_adyen_build_payment_payload( $order, $reference ),
			array(
				'paymentMethod' => array(
					'type'           => $this->payment_method_type(),
					'applepay.token' => $token,
				),
			)
		);

		return $payload;
	}

	/**
	 * Processes the payment.
	 *
	 * @param int $order_id Order ID.
	 * @return array
	 */
	public function process_payment( $order_id ) {

		parent::process_payment( $order_id );

		$order = wc_get_order( $order_id );

		$payload = $this->wkwc_adyen_build_applepay_payment_payload( $order, $order_id );

		$response = $this->checkout_action->wkwc_adyen_send_payment( $payload );

		if ( 200 === $response->status ) {
			return $this->wkwc_adyen_process_applepay_payment_result( $response, $order );
		} else {
			wc_add_notice( $response->body['message'], 'error' );

			NoticeHandler::convert_notices_to_exceptions( 'woocommerce_rest_payment_error' );
		}

		return array( 'result' => 'failure' );
	}


	/**
	 * Processes the payment result.
	 *
	 * @since 1.3.2
	 * @param object    $response Response object.
	 * @param \WC_Order $order Order object.
	 * @return array
	 */
	protected function wkwc_adyen_process_applepay_payment_result( $response, $order ) {

		$body_response = $this->adyen_helper::wkwc_adyen_obj_to_arr( $response->body );

		$result_code = isset( $body_response['resultCode'] ) ? $body_response['resultCode'] : '';
		$reference   = isset( $body_response['pspReference'] ) ? $body_response['pspReference'] : '';
		$action      = isset( $body_response['action'] ) ? $body_response['action'] : array();

		$result = array(
			'result'   => 'success',
			'redirect' => Wkwc_Adyen_Service_Util::wkwc_adyen_get_payment_return_page_url( $order, $result_code ),
		);

		$order->read_meta_data();

		$order_psp_reference = $order->get_meta( '_' . WKWC_ADN_PREFIX . '_payment_pspReference' );

		if ( $order_psp_reference !== $reference ) {
			$order->update_meta_data( '_' . WKWC_ADN_PREFIX . '_refund_pspReference', $reference );
		}

		$order->update_meta_data( '_' . WKWC_ADN_PREFIX . '_payment_resultCode', $result_code );
		$order->update_meta_data( '_' . WKWC_ADN_PREFIX . '_payment_action', $action );
		$order->save();

		if ( 'RedirectShopper' === $result_code ) {
			// redirect to process payment action via Web Component.
			$result = array(
				'result'   => 'success',
				'redirect' => add_query_arg(
					array(
						WKWC_ADN_PREFIX . '_payment_method' => $this->payment_method_type(),
						WKWC_ADN_PREFIX . '_order_id' => $order->get_id(),
						WKWC_ADN_PREFIX . '_process_order_nonce' => wp_create_nonce( 'wkwc_adyen_order_process_payment_nonce' ),
					),
					Wkwc_Adyen_Service_Util::wkwc_adyen_get_checkout_url( $order )
				),
			);
		}

		// Payment went through.
		if ( 'Authorised' === $result_code ) {
			$order->update_meta_data( '_woocommerce_' . $this->payment_method_type() . '_payment_data', $body_response );

			if ( $this->adyen_config->wkwc_adyen_is_capture_immediately() ) {
				$order->update_meta_data( '_wkwc_adyen_payment_captured', 'yes' );
				$order->set_transaction_id( $reference );
				$order->payment_complete();
				/* translators: %s: is Transaction reference */
				$order->add_order_note( sprintf( __( 'Payment was complete via Adyen (PSP Reference: %s)', 'wkwc-adyen' ), $this->adyen_helper->wkwc_adyen_get_adyen_transaction_url( $reference ) ) );
				$order->save();
			} else {
				$order->update_status( 'on-hold', __( 'Payment was successful via Adyen', 'wkwc-adyen' ) );
			}
		}

		return $result;
	}
}
