<?php
/**
 * Klarna (Pay over time)
 *
 * Payment type     : Buy Now Pay Later
 * Payment flow     : Redirect
 * Countries        : AU, AT, FI, DE, IT, NO, ES, SE, GB, US
 * Currencies       : AUD, EUR, SEK, GBP, USD
 *
 * @package WkwcAdyen
 * @version 1.2.0
 */

namespace WkwcAdyen\Includes\Payment_Methods;

// prevent direct access data leaks.
defined( 'ABSPATH' ) || exit;

use Automattic\WooCommerce\StoreApi\Utilities\NoticeHandler;

use WkwcAdyen\Helper\Common\Wkwc_Adyen_Service_Util;

use WkwcAdyen\Helper\Common\Wkwc_Adyen_Checkout_Action;

/**
 * Wkwc_Adyen_Klarna_Account class.
 */
class Wkwc_Adyen_Klarna_Account extends Wkwc_Adyen_Abstract_Gateway {
	/**
	 * $checkout_action
	 *
	 * @var Wkwc_Adyen_Checkout_Action
	 */
	public $checkout_action;

	/**
	 * Constructor of this class.
	 *
	 * @param bool $init_hooks Whether or not to initialize hooks.
	 * @since 1.1.0
	 */
	public function __construct( $init_hooks = true ) {

		parent::__construct( $init_hooks );

		$this->has_fields = false;

		if ( 'yes' !== get_option( WKWC_ADN_PREFIX . '_auto_klarna_payments' ) ) {
			$this->supports = array(
				'products',
				'refunds',
			);
		}

		$this->checkout_action = new Wkwc_Adyen_Checkout_Action();
	}

	/**
	 * List of countries where is available.
	 *
	 * @since 1.1.0
	 * @return array
	 */
	public function available_countries() {

		return array(
			'AU' => array(
				'currencies' => array( 'AUD' ),
				'recurring'  => false,
			),
			'AT' => array(
				'currencies' => array( 'EUR' ),
				'recurring'  => false,
			),
			'FI' => array(
				'currencies' => array( 'EUR' ),
				'recurring'  => false,
			),
			'DE' => array(
				'currencies' => array( 'EUR' ),
				'recurring'  => false,
			),
			'IT' => array(
				'currencies' => array( 'EUR' ),
				'recurring'  => false,
			),
			'NO' => array(
				'currencies' => array( 'NOK' ),
				'recurring'  => false,
			),
			'ES' => array(
				'currencies' => array( 'EUR' ),
				'recurring'  => false,
			),
			'SE' => array(
				'currencies' => array( 'SEK' ),
				'recurring'  => false,
			),
			'GB' => array(
				'currencies' => array( 'GBP' ),
				'recurring'  => false,
			),
			'US' => array(
				'currencies' => array( 'USD' ),
				'recurring'  => true,
			),
		);
	}

	/**
	 * Gets default payment method title.
	 *
	 * @since 1.1.0
	 * @return string
	 */
	public function get_default_title() {
		return __( 'Adyen - Klarna - Pay over time', 'wkwc-adyen' );
	}

	/**
	 * Gets default payment method description.
	 *
	 * @since 1.2.0 - show supported countries for recurring payments.
	 * @since 1.1.0
	 * @return string
	 */
	public function get_default_description() {

		$output = sprintf(
			/* translators: %s: supported country for recurring payments. */
			__( 'Spread out payments over time in easy installments. %s', 'wkwc-adyen' ),
			'<br/>' . $this->show_supported_country()
		);
		$output .= '<br/>' . $this->wkwc_adyen_show_rec_supported_country();

		return $output;
	}

	/**
	 * Gets default description set in settings.
	 *
	 * @since 1.1.0
	 * @return void|string The default description
	 */
	public function get_settings_description() {
		$this->description = ( $this->settings['description'] ) ? $this->settings['description'] : __( 'Pay Over Time with – Klarna', 'wkwc-adyen' );
		return $this->description;
	}

	/**
	 * Type of the payment method (e.g ideal, scheme. bcmc).
	 *
	 * @since 1.1.0
	 * @return string
	 */
	public function payment_method_type() {
		return 'klarna_account';
	}

	/**
	 * Returns the payment method to be used for recurring payments
	 *
	 * @since 1.1.0
	 * @return string
	 */
	public function recurring_payment_method() {
		return $this->payment_method_type();
	}

	/**
	 * Validates extra added fields.
	 *
	 * @since 1.1.0
	 * @return bool
	 */
	public function validate_fields() { // phpcs:ignore
		return parent::validate_fields();
	}

	/**
	 * Add payment fields.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function payment_fields() {

		parent::payment_fields();

		echo $this->wkwc_adyen_generate_klarna_extra_fields_html(); // phpcs:ignore
	}

	/**
	 * Generates extra fields HTML.
	 *
	 * @since 1.1.0 - added CSS class
	 * @since 1.0.0
	 * @return void - echos the HTML.
	 */
	public function wkwc_adyen_generate_klarna_extra_fields_html() {
		?>
		<div id="<?php echo esc_attr( WKWC_ADN_PREFIX . '-' . $this->payment_method_type() . '-container' ); ?>"></div>
		<input type="hidden" id="<?php echo esc_attr( 'wkwc-adyen' . $this->payment_method_type() . '_issuer' ); ?>" name="<?php echo esc_attr( WKWC_ADN_PREFIX . '_' . $this->payment_method_type() . '_issuer' ); ?>">
		<?php
	}

	/**
	 * Builds the required payment payload
	 *
	 * @since 1.1.0
	 * @param \WC_Order $order The order object.
	 * @param string    $reference The reference.
	 * @return array
	 */
	public function wkwc_adyen_build_klarna_payment_payload( \WC_Order $order, $reference ) { // phpcs:ignore
		return parent::wkwc_adyen_build_payment_payload( $order, $reference );
	}

	/**
	 * Processes the payment.
	 *
	 * \WC_Order instance to manipulate metadata
	 *
	 * @param int $order_id ID of the order.
	 * @return array
	 */
	public function process_payment( $order_id ) {

		parent::process_payment( $order_id );

		$order = wc_get_order( $order_id );

		$reference = $order_id;

		$payload = $this->wkwc_adyen_build_klarna_payment_payload( $order, $reference );

		$response = $this->checkout_action->wkwc_adyen_send_payment( $payload );

		if ( 200 === $response->status ) {
			return $this->wkwc_adyen_process_klarna_payment_result( $response, $order );
		} else {
			wc_add_notice( $response->body['message'], 'error' );

			NoticeHandler::convert_notices_to_exceptions( 'woocommerce_rest_payment_error' );
		}

		return array( 'result' => 'failure' );
	}

	/**
	 * Processes the payment result.
	 *
	 * @param object    $response Response object.
	 * @param \WC_Order $order Order object.
	 * @return array
	 */
	protected function wkwc_adyen_process_klarna_payment_result( $response, $order ) {

		$body_response = $this->adyen_helper::wkwc_adyen_obj_to_arr( $response->body );

		$result_code = isset( $body_response['resultCode'] ) ? $body_response['resultCode'] : '';
		$reference   = isset( $body_response['pspReference'] ) ? $body_response['pspReference'] : '';
		$action      = isset( $body_response['action'] ) ? $body_response['action'] : array();

		$extra_action_codes = array( 'RedirectShopper' );
		$error_codes        = array( 'Refused', 'Error', 'Cancelled' );

		$result = array(
			'result'   => 'success',
			'redirect' => Wkwc_Adyen_Service_Util::wkwc_adyen_get_payment_return_page_url( $order, $result_code ),
		);

		$order->update_meta_data( '_' . WKWC_ADN_PREFIX . '_payment_resultCode', $result_code );
		$order->update_meta_data( '_' . WKWC_ADN_PREFIX . '_payment_action', $action );
		$order->save();

		if ( in_array( $result_code, $extra_action_codes, true ) ) {
			// redirect to process payment action via Web Component.
			$result = array(
				'result'   => 'success',
				'redirect' => add_query_arg(
					array(
						WKWC_ADN_PREFIX . '_payment_method' => $this->payment_method_type(),
						WKWC_ADN_PREFIX . '_order_id' => $order->get_id(),
						WKWC_ADN_PREFIX . '_process_order_nonce' => wp_create_nonce( 'wkwc_adyen_order_process_payment_nonce' ),
					),
					Wkwc_Adyen_Service_Util::wkwc_adyen_get_checkout_url( $order )
				),
			);
		} elseif ( in_array( $result_code, $error_codes, true ) ) {
			wc_add_notice( __( 'The transaction could not be processed! Either it has been refused or an error has occurred! Please make sure you provide a valid bank account.', 'wkwc-adyen' ), 'error' );

			NoticeHandler::convert_notices_to_exceptions( 'woocommerce_rest_payment_error' );

			$result = array(
				'result' => 'failure',
			);
		} elseif ( 'Authorised' === $result_code ) {
			$order->update_meta_data( '_woocommerce_' . $this->payment_method_type() . '_payment_data', $body_response );

			if ( $this->adyen_config->wkwc_adyen_is_capture_immediately() ) {
				$order->update_meta_data( '_wkwc_adyen_payment_captured', 'yes' );
				$order->set_transaction_id( $reference );
				$order->payment_complete();
				/* translators: %s: is Transaction reference */
				$order->add_order_note( sprintf( __( 'Payment was complete via Adyen (PSP Reference: %s)', 'wkwc-adyen' ), $this->adyen_helper->wkwc_adyen_get_adyen_transaction_url( $reference ) ) );
				$order->save();
			} else {
				$order->update_status( 'on-hold', __( 'Payment was successful via Adyen', 'wkwc-adyen' ) );
			}
		}

		return $result;
	}

	/**
	 * Checks whether or not the recurring payments are supported by the country.
	 *
	 * @since 1.2.0
	 * @return bool
	 */
	public function support_recurring() {

		if ( WC()->cart && $this->get_order_total() > 0 ) {
			if ( ! empty( $this->available_countries() ) ) {
				$available_countries = $this->available_countries();
				$customer_country    = WC()->customer->get_billing_country();
				$country             = $available_countries[ $customer_country ];

				return true === $country['recurring'] ? true : false;
			}
		}

		return true;
	}

	/**
	 * Display the countries which support recurring payments.
	 *
	 * @since 1.2.0
	 * @return array
	 */
	public function wkwc_adyen_show_rec_supported_country() {

		$output    = '';
		$countries = array();

		foreach ( $this->available_countries() as $country_code => $data ) {
			$country_code = '_ANY_' === $country_code ? __( 'ANY', 'wkwc-adyen' ) : $country_code;

			if ( true === $data['recurring'] ) {
				if ( empty( $data['currencies'] ) ) {
					$countries[] = $country_code;
				} else {
					$currencies  = $data['currencies'];
					$countries[] = $country_code . ' (' . implode( ', ', $currencies ) . ')';
				}
			}
		}

		if ( ! empty( $countries ) ) {
			$output = sprintf(
				/* translators: %s: list of countries */
				__( '%1$sSupported country for recurring payments:%2$s %3$s', 'wkwc-adyen' ),
				'<b>',
				'</b>',
				implode( ', ', $countries )
			);
		}

		return $output;
	}
}
