<?php
/**
 * Ideal
 *
 * Payment type     : Online banking
 * Payment flow     : Redirect
 * Countries        : NL
 * Currencies       : EUR
 *
 * @package WkwcAdyen
 * @version 1.2.0
 */

namespace WkwcAdyen\Includes\Payment_Methods;

// prevent direct access data leaks.
defined( 'ABSPATH' ) || exit;

use Automattic\WooCommerce\StoreApi\Utilities\NoticeHandler;

use WkwcAdyen\Helper\Common\Wkwc_Adyen_Service_Util;

use WkwcAdyen\Helper\Common\Wkwc_Adyen_Checkout_Action;

/**
 * Wkwc_Adyen_Klarna_Ideal class.
 */
class Wkwc_Adyen_Klarna_Ideal extends Wkwc_Adyen_Abstract_Gateway {
	/**
	 * $checkout_action
	 *
	 * @var Wkwc_Adyen_Checkout_Action
	 */
	public $checkout_action;

	/**
	 * Constructor of this class.
	 *
	 * @param bool $init_hooks Whether or not to initialize hooks.
	 * @since 1.1.0
	 */
	public function __construct( $init_hooks = true ) {

		parent::__construct( $init_hooks );

		$this->has_fields = false;

		$this->supports = array(
			'products',
			'refunds',
		);

		$this->checkout_action = new Wkwc_Adyen_Checkout_Action();
	}

	/**
	 * List of countries where is available.
	 *
	 * @since 1.1.0
	 * @return array
	 */
	public function available_countries() {

		return array(
			'NL' => array(
				'currencies' => array( 'EUR' ),
			),
		);
	}

	/**
	 * Gets default payment method title.
	 *
	 * @since 1.0.0
	 * @return string
	 */
	public function get_default_title() {
		return __( 'Adyen - iDEAL', 'wkwc-adyen' );
	}

	/**
	 * Gets default payment method description.
	 *
	 * @since 1.1.0 - display supported countries
	 * @since 1.0.0
	 * @return string
	 */
	public function get_default_description() {
		return sprintf(
			/* translators: %s: supported country for recurring payments. */
			__( 'To enable recurring payments with iDeal, you must first activate SEPA Direct Debit functionality. %s', 'wkwc-adyen' ),
			'<br/>' . $this->show_supported_country()
		);
	}

	/**
	 * Gets default description set in settings.
	 *
	 * @since 1.0.0
	 * @return string
	 */
	public function get_settings_description() {
		$this->description = ( $this->settings['description'] ) ? $this->settings['description'] : __( 'Pay with - Ideal Pay', 'wkwc-adyen' );
		return $this->description;
	}



	/**
	 * Type of the payment method (e.g ideal, scheme. bcmc).
	 *
	 * @since 1.0.0
	 * @return string
	 */
	public function payment_method_type() {
		return 'ideal';
	}

	/**
	 * Returns the payment method to be used for recurring payments
	 *
	 * @since 1.0.0
	 * @return string
	 */
	public function recurring_payment_method() {
		return 'sepadirectdebit';
	}

	/**
	 * Adds extra fields.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function payment_fields() {

		parent::payment_fields();

		echo $this->wkwc_adyen_generate_klarna_extra_fields_html(); // phpcs:ignore
	}

	/**
	 * Generates extra fields HTML.
	 *
	 * @since 1.1.0 - added CSS class
	 * @since 1.0.0
	 * @return string - HTML markup.
	 */
	public function wkwc_adyen_generate_klarna_extra_fields_html() {
		$container_id = 'wkwc-adyen-' . esc_attr( $this->payment_method_type() ) . '-container';
		$nonce_id     = esc_attr( $this->id . '_nonce' );
		$nonce_name   = esc_attr( $this->id . '_nonce' );
		$nonce_value  = esc_attr( wp_create_nonce( $this->id . '_nonce' ) );
		$issuer_id    = 'wkwc-adyen_' . esc_attr( $this->payment_method_type() ) . '_issuer';
		$issuer_name  = esc_attr( WKWC_ADN_PREFIX . '_' . $this->payment_method_type() . '_issuer' );

		ob_start();
		?>
		<div id="<?php echo esc_attr( $container_id ); ?>"></div>
		<!-- Hidden field to store the issuer. -->
		<input type="hidden" id="<?php echo esc_attr( $nonce_id ); ?>" name="<?php echo esc_attr( $nonce_name ); ?>" value="<?php echo esc_attr( $nonce_value ); ?>">
		<input type="hidden" id="<?php echo esc_attr( $issuer_id ); ?>" name="<?php echo esc_attr( $issuer_name ); ?>">
		<?php
		return ob_get_clean();
	}

	/**
	 * Validates extra added fields.
	 *
	 * @since 1.0.0
	 * @return bool
	 */
	public function validate_fields() {

		$is_valid = parent::validate_fields();

		// Verify the nonce for security.
		if ( ! \WC_Blocks_Utils::has_block_in_page( wc_get_page_id( 'checkout' ), 'woocommerce/checkout' ) ) {
			$nonce = isset( $_POST[ $this->id . '_nonce' ] ) ? sanitize_text_field( $_POST[ $this->id . '_nonce' ] ) : '';

			if ( ! wp_verify_nonce( $nonce, $this->id . '_nonce' ) ) {
				wp_die( esc_html__( 'Nope! Security check failed!', 'wkwc-adyen' ) );
				$is_valid = false;
			}
			$issuer = isset( $_POST[ WKWC_ADN_PREFIX . '_' . $this->payment_method_type() . '_issuer' ] ) ? sanitize_text_field( $_POST[ WKWC_ADN_PREFIX . '_' . $this->payment_method_type() . '_issuer' ] ) : '';
		} else {
			$issuer = isset( $_POST[ WKWC_ADN_PREFIX . '_' . $this->payment_method_type() . '_issuer' ] ) ? sanitize_text_field( $_POST[ WKWC_ADN_PREFIX . '_' . $this->payment_method_type() . '_issuer' ] ) : '';
		}

		if ( empty( $issuer ) ) {
			$is_valid = false;
			wc_add_notice( __( 'Please select your bank account.', 'wkwc-adyen' ), 'error' );
		}

		return $is_valid;
	}

	/**
	 * Builds the required payment payload
	 *
	 * @since 1.1.0 - use parent function to get common data
	 * @since 1.0.0
	 * @param \WC_Order $order Order object.
	 * @param string    $reference Payment reference.
	 * @return array
	 */
	public function wkwc_adyen_build_klarna_payment_payload( \WC_Order $order, $reference ) {

		// Verify the nonce for security.
		if ( ! \WC_Blocks_Utils::has_block_in_page( wc_get_page_id( 'checkout' ), 'woocommerce/checkout' ) ) {
			$nonce = isset( $_POST[ $this->id . '_nonce' ] ) ? sanitize_text_field( $_POST[ $this->id . '_nonce' ] ) : '';
			if ( ! wp_verify_nonce( $nonce, $this->id . '_nonce' ) ) {
				wp_die( esc_html__( 'Nope! Security check failed!', 'wkwc-adyen' ) );
				return array();
			}
			$issuer = isset( $_POST[ WKWC_ADN_PREFIX . '_' . $this->payment_method_type() . '_issuer' ] ) ? sanitize_text_field( $_POST[ WKWC_ADN_PREFIX . '_' . $this->payment_method_type() . '_issuer' ] ) : '';
		} else {
			$issuer = isset( $_POST[ WKWC_ADN_PREFIX . '_' . $this->payment_method_type() . '_issuer' ] ) ? sanitize_text_field( $_POST[ WKWC_ADN_PREFIX . '_' . $this->payment_method_type() . '_issuer' ] ) : '';
		}

		$payload = parent::wkwc_adyen_build_payment_payload( $order, $reference );

		$payload['paymentMethod']['issuer'] = $issuer;

		return $payload;
	}

	/**
	 * Processes the payment.
	 *
	 * @since 1.1.0 - replace `_#subscription#_` with `-S`
	 * @since 1.0.7 - use \WC_Order instance to manipulate metadata
	 * @since 1.0.0
	 * @param int $order_id Order ID.
	 * @return array
	 */
	public function process_payment( $order_id ) {

		parent::process_payment( $order_id );

		$order     = wc_get_order( $order_id );
		$reference = $order_id;
		$payload   = $this->wkwc_adyen_build_klarna_payment_payload( $order, $reference );

		$response = $this->checkout_action->wkwc_adyen_send_payment( $payload );

		if ( 200 === $response->status ) {
			return $this->wkwc_adyen_process_klarna_payment_result( $response, $order );
		} else {
			wc_add_notice( $response->body['message'], 'error' );

			NoticeHandler::convert_notices_to_exceptions( 'woocommerce_rest_payment_error' );
		}

		return array( 'result' => 'failure' );
	}

	/**
	 * Processes the payment result.
	 *
	 * @param object    $response Response object.
	 * @param \WC_Order $order Order object.
	 * @return array
	 */
	protected function wkwc_adyen_process_klarna_payment_result( $response, $order ) {
		$body_response = $this->adyen_helper::wkwc_adyen_obj_to_arr( $response->body );

		$result_code = isset( $body_response['resultCode'] ) ? $body_response['resultCode'] : '';
		$reference   = isset( $body_response['pspReference'] ) ? $body_response['pspReference'] : '';
		$action      = isset( $body_response['action'] ) ? $body_response['action'] : array();

		$extra_action_codes = array( 'RedirectShopper' );
		$error_codes        = array( 'Refused', 'Error', 'Cancelled' );

		$result = array(
			'result'   => 'success',
			'redirect' => Wkwc_Adyen_Service_Util::wkwc_adyen_get_payment_return_page_url( $order, $result_code ),
		);

		$order->update_meta_data( '_' . WKWC_ADN_PREFIX . '_payment_resultCode', $result_code );
		$order->update_meta_data( '_' . WKWC_ADN_PREFIX . '_payment_action', $action );
		$order->save();

		if ( in_array( $result_code, $extra_action_codes, true ) ) {
			// redirect to process payment action via Web Component.
			$result = array(
				'result'   => 'success',
				'redirect' => add_query_arg(
					array(
						WKWC_ADN_PREFIX . '_payment_method' => $this->payment_method_type(),
						WKWC_ADN_PREFIX . '_order_id' => $order->get_id(),
						WKWC_ADN_PREFIX . '_process_order_nonce' => wp_create_nonce( 'wkwc_adyen_order_process_payment_nonce' ),
					),
					Wkwc_Adyen_Service_Util::wkwc_adyen_get_checkout_url( $order )
				),
			);
		} elseif ( in_array( $result_code, $error_codes, true ) ) {
			wc_add_notice( __( 'The transaction could not be processed! Either it has been refused or an error has occurred! Please make sure you provide a valid bank account.', 'wkwc-adyen' ), 'error' );

			NoticeHandler::convert_notices_to_exceptions( 'woocommerce_rest_payment_error' );

			$result = array(
				'result' => 'failure',
			);
		} elseif ( 'Authorised' === $result_code ) {
			$order->update_meta_data( '_woocommerce_' . $this->payment_method_type() . '_payment_data', $body_response );

			if ( $this->adyen_config->wkwc_adyen_is_capture_immediately() ) {
				$order->update_meta_data( '_wkwc_adyen_payment_captured', 'yes' );
				$order->set_transaction_id( $reference );
				$order->payment_complete();
				/* translators: %s: is Transaction reference */
				$order->add_order_note( sprintf( __( 'Payment was complete via Adyen (PSP Reference: %s)', 'wkwc-adyen' ), $this->adyen_helper->wkwc_adyen_get_adyen_transaction_url( $reference ) ) );
				$order->save();
			} else {
				$order->update_status( 'on-hold', __( 'Payment was successful via Adyen', 'wkwc-adyen' ) );
			}
		}

		return $result;
	}
}
