<?php
/**
 * This file handles the Webkul's plugins real time update notification functions from Envato.
 *
 * @version 1.0.0
 *
 * @package WK Plugin Updates
 */

defined( 'ABSPATH' ) || exit(); // Exit if access directly.

if ( ! class_exists( 'WKPU_Envato_Functions' ) ) {
	/**
	 * WKPU_Envato_Functions class.
	 */
	class WKPU_Envato_Functions {
		/**
		 * Autoload constructor.
		 */
		public function __construct() {
		}

		/**
		 * Check if Envato is connected.
		 */
		public function envato_connected() {
			$authorize = $this->authorize_token_permissions();

			return 'success' === $authorize;
		}

		/**
		 * Authorize token permissions.
		 *
		 * @param string $token Authorization token.
		 *
		 * @return string
		 */
		public function authorize_token_permissions( $token = '' ) {
			$notice = 'success';

			if ( empty( $token ) ) {
				$notice = get_site_transient( WKPU_UPDATE_OPTION_NAME . '_connected_status' );

				if ( ! empty( $notice ) ) {
					return $notice;
				}
				$notice = 'success';
			}

			self::log( 'Authorization token status: ' . $notice . ' Token: ' . $token );

			$args     = empty( $token ) ? array() : array( 'headers' => $this->request_headers( $token ) );
			$response = $this->request( 'https://api.envato.com/whoami', $args );

			if ( is_wp_error( $response ) && ( $response->get_error_code() === 'http_error' || $response->get_error_code() === 500 ) ) {
				$this->store_additional_error_debug_information( 'An error occured checking token permissions', $response->get_error_message(), $response->get_error_data() );
				$notice = 'http_error';
			} elseif ( is_wp_error( $response ) || ! isset( $response['scopes'] ) || ! is_array( $response['scopes'] ) ) {
				$this->store_additional_error_debug_information( 'No scopes found in API response message', $response->get_error_message(), $response->get_error_data() );
				$notice = 'error';
			} else {
				$minimum_scopes = $this->get_required_permissions();
				$maximum_scopes = array( 'default' => 'Default' ) + $minimum_scopes;

				foreach ( $minimum_scopes as $required_scope => $required_scope_name ) {
					if ( ! in_array( $required_scope, $response['scopes'], true ) ) {
						// The scope minimum required scope doesn't exist.
						$this->store_additional_error_debug_information( 'Could not find required API permission scope in output.', $required_scope );
						$notice = 'missing-permissions';
					}
				}
				foreach ( $response['scopes'] as $scope ) {
					if ( ! isset( $maximum_scopes[ $scope ] ) ) {
						// The available scope is outside our maximum bounds.
						$this->store_additional_error_debug_information( 'Found too many permissions on token.', $scope );
						$notice = 'too-many-permissions';
					}
				}
			}

			if ( 'success' === $notice ) {
				delete_site_transient( WKPU_UPDATE_OPTION_NAME . '_error_information' );
			}

			set_site_transient( WKPU_UPDATE_OPTION_NAME . '_connected_status', $notice, HOUR_IN_SECONDS );

			$this->authorize_plugins();

			return $notice;
		}

		/**
		 * Store additional error information in transient so users can self debug.
		 *
		 * @param string $title Title of the error.
		 * @param string $message Error message.
		 * @param array  $data Error Data.
		 */
		public function store_additional_error_debug_information( $title, $message = '', $data = array() ) {
			set_site_transient(
				WKPU_UPDATE_OPTION_NAME . '_error_information',
				array(
					'title'   => $title,
					'message' => $message,
					'data'    => $data,
				),
				2 * MINUTE_IN_SECONDS
			);
		}

		/**
		 * Query the Envato API.
		 *
		 * @uses wp_remote_get() To perform an HTTP request.
		 *
		 * @since 1.0.0
		 *
		 * @param  string $url API request URL, including the request method, parameters, & file type.
		 * @param  array  $args The arguments passed to `wp_remote_get`.
		 *
		 * @return array|WP_Error The HTTP response.
		 */
		public function request( $url, $args = array() ) {
			$defaults = array(
				'sslverify' => true,
				'headers'   => $this->request_headers(),
				'timeout'   => 14,
			);

			$args = wp_parse_args( $args, $defaults );

			$token = trim( str_replace( 'Bearer', '', $args['headers']['Authorization'] ) );
			if ( empty( $token ) || strlen( $token ) < 5 ) {
				return new WP_Error( 'api_token_error', __( 'An valid API token is required.', 'wkpu_updates' ) );
			}

			$debugging_information = array(
				'request_url' => $url,
			);

			// Make an API request.
			$response = wp_remote_get( esc_url_raw( $url ), $args );

			// Check the response code.
			$response_code    = wp_remote_retrieve_response_code( $response );
			$response_message = wp_remote_retrieve_response_message( $response );

			$debugging_information['response_code']   = $response_code;
			$debugging_information['response_cf_ray'] = wp_remote_retrieve_header( $response, 'cf-ray' );
			$debugging_information['response_server'] = wp_remote_retrieve_header( $response, 'server' );

			if ( ! empty( $response->errors ) && isset( $response->errors['http_request_failed'] ) ) {
				// API connectivity issue, inject notice into transient with more details.
				$options = get_option( WKPU_UPDATE_OPTION_NAME . '_api_options', array() );

				if ( empty( $options['notices'] ) ) {
					$options['notices'] = array();
				}
				$options['notices']['http_error'] = current( $response->errors['http_request_failed'] );
				update_option( WKPU_UPDATE_OPTION_NAME . '_api_options', $options );
				return new \WP_Error( 'http_error', esc_html( current( $response->errors['http_request_failed'] ) ), $debugging_information );
			}

			if ( 200 !== $response_code && ! empty( $response_message ) ) {
				return new \WP_Error( $response_code, $response_message, $debugging_information );
			} elseif ( 200 !== $response_code ) {
				return new \WP_Error( $response_code, __( 'An unknown API error occurred.', 'wkpu_updates' ), $debugging_information );
			} else {
				$return = json_decode( wp_remote_retrieve_body( $response ), true );
				if ( null === $return ) {
					return new \WP_Error( 'api_error', __( 'An unknown API error occurred.', 'wkpu_updates' ), $debugging_information );
				}
				return $return;
			}
		}

		/**
		 * Adding request headers.
		 *
		 * @param string $token Authorization token.
		 */
		private function request_headers( $token = '' ) {
			$token = empty( $token ) ? get_option( 'wkwc_envato_user_token', false ) : $token;

			$user_agent = array( 'User-Agent' => 'WordPress - Envato Market ' . WKPU_UPDATE_VERSION );
			$headers    = array_merge( $user_agent, array( 'Authorization' => 'Bearer ' . trim( $token ) ) );
			return $headers;
		}

		/**
		 * Defers building the API download url until the last responsible moment to limit file requests.
		 *
		 * Filter the package options before running an update.
		 *
		 * @param array $options {
		 *    Options used by the upgrader.
		 *
		 * @type string $package Package for update.
		 * @type string $destination Update location.
		 * @type bool   $clear_destination Clear the destination resource.
		 * @type bool   $clear_working Clear the working resource.
		 * @type bool   $abort_if_destination_exists Abort if the Destination directory exists.
		 * @type bool   $is_multi Whether the upgrader is running multiple times.
		 * @type array  $hook_extra Extra hook arguments.
		 * }
		 */
		public function maybe_deferred_download( $options ) {
			$package = $options['package'];
			if ( false !== strrpos( $package, 'deferred_download' ) && false !== strrpos( $package, 'item_id' ) ) {
				parse_str( wp_parse_url( $package, PHP_URL_QUERY ), $vars );
				if ( $vars['item_id'] ) {
					$options['package'] = $this->download_plugin_url( $vars['item_id'] );
				}
			}
			return $options;
		}

		/**
		 * Download Plugin Package URL.
		 *
		 * @param string $item_id Webkul Plugin id.
		 */
		public function download_plugin_url( $item_id ) {
			$api_domain = 'https://api.envato.com';
			$api_path   = '/v3/market/buyer/download';

			$url      = $api_domain . $api_path . '?item_id=' . $item_id . '&shorten_url=true';
			$response = $this->request( $url );

			if ( is_wp_error( $response ) || empty( $response ) || ! empty( $response['error'] ) ) {
				return false;
			}

			if ( ! empty( $response['wordpress_plugin'] ) ) {
				return $response['wordpress_plugin'];
			}
			return false;
		}

		/**
		 * We want to stop certain popular 3rd party scripts from blocking the update process by
		 * adjusting the plugin name slightly so the 3rd party plugin checks stop.
		 *
		 * Currently works for: Visual Composer.
		 *
		 * @param string $reply Package URL.
		 * @param string $package Package URL.
		 * @param object $updater Updater Object.
		 *
		 * @return string $reply    New Package URL.
		 * @since 2.0.0
		 */
		public function upgrader_pre_download( $reply, $package, $updater ) {
			if ( strpos( $package, 'marketplace.envato.com/short-dl' ) !== false ) {
				if ( isset( $updater->skin->plugin_info ) && ! empty( $updater->skin->plugin_info['Name'] ) ) {
					$updater->skin->plugin_info['Name'] = $updater->skin->plugin_info['Name'] . '.';
				} else {
					$updater->skin->plugin_info = array( 'Name' => 'Name' );
				}
			}

			return $reply;
		}

		/**
		 * Check plugins are authorized and downloadable.
		 *
		 * @return bool|null
		 */
		public function authorize_plugins() {
			$api_domain = 'https://api.envato.com';
			$api_path   = '/v3/market/buyer/list-purchases';

			$url      = $api_domain . $api_path . '?filter_by=wordpress-plugins';
			$response = $this->request( $url );

			$plugins = array();

			if ( is_wp_error( $response ) || empty( $response ) || empty( $response['results'] ) ) {
				return $plugins;
			}

			foreach ( $response['results'] as $plugin ) {
				$purchased = $this->prepare_purchased_plugin_data( $plugin );

				if ( ! empty( $purchased ) ) {
					$plugins[] = $purchased;
				}
			}

			if ( empty( $plugins ) ) {
				delete_site_transient( WKPU_UPDATE_OPTION_NAME . '_purchased_plugins' );
			} else {
				set_site_transient( WKPU_UPDATE_OPTION_NAME . '_purchased_plugins', $plugins, HOUR_IN_SECONDS );
				update_option( WKPU_UPDATE_OPTION_NAME . '_purchased_plugins', $plugins );
			}
		}

		/**
		 * Prepare purchased plugin data.
		 *
		 * @param array $plugin_data An array of API response.
		 *
		 * @return array
		 */
		public function prepare_purchased_plugin_data( $plugin_data ) {
			$plugin_item    = empty( $plugin_data['item'] ) ? array() : $plugin_data['item'];
			$plugin_item_id = empty( $plugin_item['id'] ) ? '' : $plugin_item['id'];

			if ( empty( $plugin_item_id ) ) {
				return array();
			}

			$plugin_normalized = array(
				'id'              => $plugin_item_id,
				'plugin_name'     => empty( $plugin_item['wordpress_plugin_metadata']['plugin_name'] ) ? '' : $plugin_item['wordpress_plugin_metadata']['plugin_name'],
				'version'         => empty( $plugin_item['wordpress_plugin_metadata']['version'] ) ? '' : $plugin_item['wordpress_plugin_metadata']['version'],
				'description'     => self::remove_non_unicode( strip_tags( $plugin_item['wordpress_plugin_metadata']['description'] ) ),
				'url'             => empty( $plugin_item['url'] ) ? '' : $plugin_item['url'],
				'author'          => empty( $plugin_item['author_username'] ) ? '' : $plugin_item['author_username'],
				'thumbnail_url'   => empty( $plugin_item['thumbnail_url'] ) ? '' : $plugin_item['thumbnail_url'],
				'landscape_url'   => empty( $plugin_item['previews']['landscape_preview']['landscape_url'] ) ? '' : $plugin_item['previews']['landscape_preview']['landscape_url'],
				'number_of_sales' => empty( $plugin_item['number_of_sales'] ) ? '' : $plugin_item['number_of_sales'],
				'updated_at'      => empty( $plugin_item['updated_at'] ) ? '' : $plugin_item['updated_at'],
				'rating'          => empty( $plugin_item['rating'] ) ? '' : $plugin_item['rating'],

				'amount'          => empty( $plugin_data['amount'] ) ? '' : $plugin_data['amount'],
				'code'            => empty( $plugin_data['code'] ) ? '' : $plugin_data['code'],
				'sold_at'         => empty( $plugin_data['sold_at'] ) ? '' : $plugin_data['sold_at'],
				'license'         => empty( $plugin_data['license'] ) ? '' : $plugin_data['license'],
				'support_amount'  => empty( $plugin_data['support_amount'] ) ? '' : $plugin_data['support_amount'],
				'supported_until' => empty( $plugin_data['supported_until'] ) ? '' : $plugin_data['supported_until'],
			);

			// No main thumbnail in API response, so we grab it from the preview array.
			if ( empty( $plugin_normalized['landscape_url'] ) && ! empty( $plugin_item['previews'] ) && is_array( $plugin_item['previews'] ) ) {
				foreach ( $plugin_item['previews'] as $possible_preview ) {
					if ( ! empty( $possible_preview['landscape_url'] ) ) {
						$plugin_normalized['landscape_url'] = $possible_preview['landscape_url'];
						break;
					}
				}
			}
			if ( empty( $plugin_normalized['thumbnail_url'] ) && ! empty( $plugin_item['previews'] ) && is_array( $plugin_item['previews'] ) ) {
				foreach ( $plugin_item['previews'] as $possible_preview ) {
					if ( ! empty( $possible_preview['icon_url'] ) ) {
						$plugin_normalized['thumbnail_url'] = $possible_preview['icon_url'];
						break;
					}
				}
			}

			return $plugin_normalized;
		}

		/**
		 * Remove all non unicode characters in a string.
		 *
		 * @param string $retval The string to fix.
		 *
		 * @return string
		 */
		private static function remove_non_unicode( $retval ) {
			return preg_replace( '/[\x00-\x1F\x80-\xFF]/', '', $retval );
		}

		/**
		 * Get Plugin item data.
		 *
		 * @param string $item_id Plugin item id.
		 *
		 * @return array Plugin item data.
		 */
		public function get_plugin_item_data( $item_id ) {
			$api_domain = 'https://api.envato.com';
			$api_path   = '/v3/market/catalog/item';

			$url      = $api_domain . $api_path . '?id=' . $item_id;
			$response = $this->request( $url );

			if ( is_wp_error( $response ) || empty( $response ) ) {
				return array();
			}
			return $response;
		}

		/**
		 * Get the required API permissions for creating envato token.
		 *
		 * @return array
		 */
		public function get_required_permissions() {
			return apply_filters(
				'wkwc_envato_token_required_permissions',
				array(
					'default'           => 'View and search Envato sites',
					'purchase:download' => 'Download your purchased items',
					'purchase:list'     => 'List purchases you\'ve made',
				)
			);
		}

		/**
		 * Log function for debugging.
		 *
		 * @param mixed  $message Message string or array.
		 * @param array  $context Additional parameter, like file name 'source'.
		 * @param string $level One of the following:
		 *     'emergency': System is unusable.
		 *     'alert': Action must be taken immediately.
		 *     'critical': Critical conditions.
		 *     'error': Error conditions.
		 *     'warning': Warning conditions.
		 *     'notice': Normal but significant condition.
		 *     'info': Informational messages.
		 *     'debug': Debug-level messages.
		 */
		public static function log( $message, $context = array(), $level = 'info' ) {
			if ( function_exists( 'wc_get_logger' ) ) {
				$log_enabled = apply_filters( 'wkpu_updates_is_log_enabled', true );

				if ( $log_enabled ) {
					$source            = ( is_array( $context ) && ! empty( $context['source'] ) ) ? $context['source'] : 'wkpu_updates';
					$context['source'] = $source;
					$logger            = wc_get_logger();
					$current_user_id   = get_current_user_id();

					$in_action = wp_sprintf( ( /* translators: %s current user id */ esc_html__( 'User in action: %s: ', 'wkpu_updates' ) ), $current_user_id );
					$message   = $in_action . $message;

					$logger->log( $level, $message, $context );
				}
			}
		}
	}
}
