<?php
/**
 * This file handles the Webkul's plugins real time update notifications.
 *
 * @version 1.0.0
 *
 * @package WK Plugin Updates
 */

defined( 'ABSPATH' ) || exit(); // Exit if access directly.

if ( ! class_exists( 'WKPU_Updates_Functions' ) ) {
	/**
	 * Including Envato function class.
	 */
	if ( ! class_exists( 'WKPU_Envato_Functions' ) ) {
		require_once __DIR__ . '/class-wkpu-envato-functions.php';
	}

	/**
	 * WKPU_Updates class.
	 */
	class WKPU_Updates_Functions extends WKPU_Envato_Functions {
		/**
		 * Background process to update plugin meta.
		 *
		 * @var WKPU_UPDATES_Background_Plugins
		 */
		public static $background_plugin_process;

		/**
		 * Autoload constructor.
		 */
		public function __construct() {
		}

		/**
		 * Initiating background processing.
		 *
		 * @return void
		 */
		public function wkpu_updates_init_background_indexer() {
			if ( ! class_exists( 'WKPU_UPDATES_Background_Plugins' ) ) {
				require_once __DIR__ . '/helper/class-wkpu-updates-background-plugins.php';
			}

			if ( class_exists( 'WKPU_UPDATES_Background_Plugins' ) ) {
				self::$background_plugin_process = new \WKPU_UPDATES_Background_Plugins();
			}
		}

		/**
		 * Adding extra plugin headers.
		 *
		 * @param array $headers Plugin headers.
		 *
		 * @hooked 'extra_plugin_headers' filter hook.
		 *
		 * @return array
		 */
		public function wkpu_updates_extra_headers( $headers ) {
			array_push( $headers, 'WKPU_Updates', 'WKLWDT' );

			return $headers;
		}

		/**
		 * Loading WKPU text domain.
		 *
		 * @return void
		 */
		public function wkpu_updates_load_textdomain() {
			load_plugin_textdomain( 'wkpu_updates', false, plugin_basename( __DIR__ ) . '/languages' );

			if ( $this->envato_connected() ) {
				add_action( 'upgrader_package_options', array( $this, 'maybe_deferred_download' ), 9 );
				add_filter( 'upgrader_pre_download', array( $this, 'upgrader_pre_download' ), 2, 4 );
			}
		}

		/**
		 * Loading assets.
		 *
		 * @return void
		 */
		public function wkpu_updates_admin_enqueue_scripts() {
			if ( in_array( get_current_screen()->id, array( 'plugins', 'plugin-install', 'webkul-wc-addons_page_wkwc-addons-license', 'marketplace_page_wk-marketplace-license', 'point-of-sale_page_wc-pos-licence', 'webkul-wp-addons_page_wkwp-addons-license' ), true ) ) {
				$suffix     = ( defined( 'WKWC_DEV' ) && true === WKWC_DEV ) ? '' : '.min';
				$asset_path = ( defined( 'WKWC_DEV' ) && true === WKWC_DEV ) ? 'src' : 'assets';
				wp_enqueue_style( 'wkpu-updates-style', WKPU_UPDATE_DIR_URL . $asset_path . '/css/wkpu-updates' . $suffix . '.css', array(), WKPU_UPDATE_VERSION, 'all' );
			}
		}

		/**
		 * Collecting plugins data.
		 *
		 * @hooked 'plugins_loaded' action hook.
		 *
		 * @return void
		 */
		public function wkpu_updates_collect_webkul_plugins_every_six_hour() {
			$force_clean_transient = filter_input( INPUT_GET, 'wkwc_pu_clean', FILTER_SANITIZE_NUMBER_INT );

			if ( 200 === intval( $force_clean_transient ) && current_user_can( 'manage_options' ) ) {
				delete_site_transient( WKPU_UPDATE_OPTION_NAME . '_wk_plugins_data' );
			}

			$wk_plugins = $this->wkpu_get_plugin_data( 'transient' );

			if ( empty( $wk_plugins ) ) {
				// Check if get_plugins() function exists. This is required on the front end of the site, since it is in a file that is normally only loaded in the admin.
				if ( ! function_exists( 'get_plugins' ) ) {
					require_once ABSPATH . 'wp-admin/includes/plugin.php';
				}

				$all_plugins = get_plugins();
				$wkpu_data   = array();

				foreach ( $all_plugins as $plugin_file => $data ) {
					if ( 'webkul' === strtolower( $data['Author'] ) ) {
						$item_id = $this->wkpu_updates_get_item_id( $data );

						if ( ! empty( $item_id ) && is_numeric( $item_id ) && ! array_key_exists( $item_id, $wkpu_data ) ) {
							$wkpu_data[ $item_id ] = array(
								'basename'  => $plugin_file,
								'installed' => empty( $data['Version'] ) ? '' : $data['Version'],
								'tested'    => empty( $data['TestedWP'] ) ? get_bloginfo( 'version' ) : $data['TestedWP'],
								'requires'  => empty( $data['RequiresWP'] ) ? get_bloginfo( 'version' ) : $data['RequiresWP'],
							);
						}
					}
				}

				set_site_transient( WKPU_UPDATE_OPTION_NAME . '_wk_plugins_data', $wkpu_data, 6 * HOUR_IN_SECONDS );
			}
		}

		/**
		 * Updating all wk plugins meta in option table twice daily via wp schedule event.
		 *
		 * @return void|false
		 *
		 * @hooked 'admin_footer' action hook.
		 */
		public function wkpu_updates_plugins_meta_data() {
			$wk_plugins = $this->wkpu_get_plugin_data( 'transient' );

			if ( ! empty( $wk_plugins ) && is_array( $wk_plugins ) ) {
				$totals       = get_site_transient( WKPU_UPDATE_OPTION_NAME . '_totals' );
				$pushed_items = $totals['pushed'] ?? array();

				$identifier = self::$background_plugin_process->get_identifier();
				$identifier = $identifier['action'] ?? '';

				$get_background_clean = filter_input( INPUT_GET, 'wkwc_pu_clean_background', FILTER_SANITIZE_FULL_SPECIAL_CHARS );

				if ( in_array( $get_background_clean, array( 'yes', 'restart' ), true ) ) {
					self::$background_plugin_process->kill_process();

					if ( 'restart' === $get_background_clean ) {
						delete_site_transient( WKPU_UPDATE_OPTION_NAME . '_wk_plugins_data' );
						delete_site_transient( WKPU_UPDATE_OPTION_NAME . '_totals' );
					}
				}

				$is_running = self::$background_plugin_process->is_running();

				if ( $is_running && self::$background_plugin_process::wkpu_updates_background_process_needs_recovery( $identifier ) ) {
					self::$background_plugin_process->kill_process();
				}

				$is_running = self::$background_plugin_process->is_running();

				WKPU_Updates::log( ' Identifier: ' . $identifier . ' Is Running: ' . $is_running . ' Clean background process: ' . $get_background_clean );

				if ( $is_running ) {
					return false;
				}

				foreach ( $wk_plugins as $item_id => $plugin_data ) {
					if ( in_array( $item_id, $pushed_items, true ) ) {
						continue;
					}
					$item_updated_at = empty( $plugin_data['item_updated_at'] ) ? '' : $plugin_data['item_updated_at'];

					$diff_days = 0;
					$diff_hour = 0;
					$diff_min  = 0;

					if ( ! empty( $item_updated_at ) ) {
						$last_date        = new DateTime( $item_updated_at );
						$current_datetime = new DateTime( gmdate( 'Y-m-d H:i:s' ) );

						$diff      = $last_date->diff( $current_datetime );
						$diff_days = empty( $diff->days ) ? 0 : intval( $diff->days );
						$diff_hour = empty( $diff->h ) ? 0 : intval( $diff->h );
						$diff_min  = empty( $diff->i ) ? 0 : intval( $diff->i );
					}

					WKPU_Updates::log( 'Item id ' . $item_id . ' Updated at: ' . $item_updated_at . ' Diff days: ' . $diff_days . ' Diff Hours: ' . $diff_hour . ' Diff Min: ' . $diff_min );

					if ( empty( $item_updated_at ) || $diff_days > 0 || $diff_hour > 5 ) {
						if ( count( $pushed_items ) < 3 && ! empty( $item_updated_at ) ) {
							WKPU_Updates::log( 'Last update: ' . print_r( $last_date, true ) );
							WKPU_Updates::log( 'Current GMT date and time: ' . print_r( $current_datetime, true ) );
							WKPU_Updates::log( 'Diff: ' . print_r( $diff, true ) );
						}
						$pushed_items[] = $item_id;

						$item_data = array(
							'item_id'     => $item_id,
							'plugin_data' => $plugin_data,
							'update_obj'  => $this,
						);

						WKPU_Updates::log( 'Pushing item: ' . $item_id . ' to background processing' );

						self::$background_plugin_process->push_to_queue( $item_data );
					}
				}

				if ( ! empty( $pushed_items ) ) {
					self::$background_plugin_process->save()->dispatch();

					$pushed_count = count( $pushed_items );

					$totals = array(
						'pushed'       => $pushed_items,
						'pushed_count' => $pushed_count,
						'completed'    => array(),
					);

					set_site_transient( WKPU_UPDATE_OPTION_NAME . '_totals', $totals, 10 * MINUTE_IN_SECONDS );

					WKPU_Updates::log( 'Dispatching ' . $pushed_count . ' items for update are: ' . print_r( $pushed_items, true ) );
				}
			}
		}

		/**
		 * Update plugin meta information.
		 *
		 * @param string $item_id Item id.
		 * @param array  $plugin_data Plugin data.
		 */
		public function update_plugin_meta( $item_id, $plugin_data ) {
			if ( $this->envato_connected() ) {
				$api_data = $this->get_plugin_item_data( $item_id );

				return $this->prepare_envato_plugin_data( $api_data, $plugin_data );
			}

			$url = 'https://wpdemo.webkul.com/wk-extensions/data-with-desc.php?item_id=' . $item_id;

			$response      = wp_remote_get( $url, array( 'sslverify' => false ) );
			$response_code = wp_remote_retrieve_response_code( $response );

			if ( 200 === $response_code ) {
				// Get the response body.
				$response_body = wp_remote_retrieve_body( $response );

				if ( ! empty( $response_body ) ) {
					// Process the API response.
					$api_data = json_decode( $response_body, true );
					$api_data = empty( $api_data[0] ) ? array() : $api_data[0];

					$plugin_data = $this->prepare_envato_plugin_data( $api_data, $plugin_data );
				}
			}

			return $plugin_data;
		}

		/**
		 * Prepare final plugin data to save in transient.
		 *
		 * @param array $api_data API data .
		 * @param array $plugin_data Plugin data.
		 *
		 * @return array
		 */
		public function prepare_envato_plugin_data( $api_data, $plugin_data ) {
			$meta_data = empty( $api_data['wordpress_plugin_metadata'] ) ? array() : $api_data['wordpress_plugin_metadata'];

			$plugin_data['plugin_name'] = empty( $meta_data['plugin_name'] ) ? '' : $meta_data['plugin_name'];
			$plugin_data['author']      = empty( $meta_data['author'] ) ? '' : $meta_data['author'];
			$plugin_data['version']     = empty( $meta_data['version'] ) ? '' : $meta_data['version'];
			$plugin_data['description'] = empty( $meta_data['description'] ) ? '' : $meta_data['description'];

			$plugin_data['url']           = empty( $api_data['url'] ) ? '' : $api_data['url'];
			$plugin_data['landscape_url'] = empty( $api_data['previews']['landscape_preview']['landscape_url'] ) ? '' : $api_data['previews']['landscape_preview']['landscape_url'];
			$plugin_data['thumbnail_url'] = empty( $api_data['thumbnail_url'] ) ? '' : $api_data['thumbnail_url'];

			$plugin_data['rating']          = empty( $api_data['rating'] ) ? '' : $api_data['rating'];
			$plugin_data['rating_count']    = empty( $api_data['rating_count'] ) ? '' : $api_data['rating_count'];
			$plugin_data['updated_at']      = empty( $api_data['updated_at'] ) ? '' : $api_data['updated_at'];
			$plugin_data['number_of_sales'] = empty( $api_data['number_of_sales'] ) ? '' : $api_data['number_of_sales'];
			$plugin_data['changelog']       = $this->parse_description_for_changelog( $api_data, $plugin_data['url'] );

			// No main thumbnail in API response, so we grab it from the preview array.
			if ( empty( $plugin_data['landscape_url'] ) && ! empty( $api_data['previews'] ) && is_array( $api_data['previews'] ) ) {
				foreach ( $api_data['previews'] as $possible_preview ) {
					if ( ! empty( $possible_preview['landscape_url'] ) ) {
						$plugin_data['landscape_url'] = $possible_preview['landscape_url'];
						break;
					}
				}
			}

			if ( empty( $plugin_data['thumbnail_url'] ) && ! empty( $api_data['previews'] ) && is_array( $api_data['previews'] ) ) {
				foreach ( $api_data['previews'] as $possible_preview ) {
					if ( ! empty( $possible_preview['icon_url'] ) ) {
						$plugin_data['thumbnail_url'] = $possible_preview['icon_url'];
						break;
					}
				}
			}

			$plugin_data['item_updated_at'] = gmdate( 'Y-m-d H:i:s' );

			return $plugin_data;
		}

		/**
		 * Get item id.
		 *
		 * @param array $plugin_data Plugin data.
		 *
		 * @return string
		 */
		public function wkpu_updates_get_item_id( $plugin_data ): string {
			$author = empty( $plugin_data['Author'] ) ? '' : strtolower( $plugin_data['Author'] );

			if ( ! empty( $plugin_data['AuthorName'] ) ) {
				$author = empty( $plugin_data['AuthorName'] ) ? '' : strtolower( $plugin_data['AuthorName'] );
			}

			if ( 'webkul' !== $author ) {
				return '';
			}

			if ( ! empty( $plugin_data['item_id'] ) ) {
				return $plugin_data['item_id'];
			}

			$parsed_url = empty( $plugin_data['PluginURI'] ) ? array() : wp_parse_url( $plugin_data['PluginURI'] );
			$path       = $parsed_url['path'] ?? '';
			$item_arr   = empty( $path ) ? '' : explode( '/', $path );

			$item_id       = empty( $item_arr ) ? '' : end( $item_arr );
			$final_item_id = ( empty( $item_id ) && ! empty( $item_arr ) ) ? $item_arr[ count( $item_arr ) - 2 ] : $item_id;

			return is_numeric( $final_item_id ) ? $final_item_id : '';
		}

		/**
		 * Add a Webkul Marketplace tab.
		 *
		 * @param array $tabs Plugins list tabs.
		 *
		 * @return array
		 */
		public function wkpu_updates_add_plugin_install_tab( $tabs ): array {
			$tabs = empty( $tabs ) ? array() : $tabs;

			$tabs['webkul'] = esc_html__( 'Webkul', 'wkpu_updates' );

			return $tabs;
		}

		/**
		 * Open the extensions when the user clicks on the webkul link in the plugin installer.
		 */
		public function wkpu_updates_webkul_plugin_install_callback() {
			$tab = filter_input( INPUT_GET, 'tab', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
			if ( 'webkul' === $tab ) {
				$ext_url = menu_page_url( 'wkwc-addons-extensions', false );
				$ext_url = empty( $ext_url ) ? menu_page_url( 'wk-marketplace-extensions', false ) : $ext_url;
				$ext_url = empty( $ext_url ) ? menu_page_url( 'wc-pos-extensions', false ) : $ext_url;
				$ext_url = empty( $ext_url ) ? menu_page_url( 'wkwp-addons-extensions', false ) : $ext_url;

				if ( ! empty( $ext_url ) ) {
					wp_safe_redirect( $ext_url );
					exit;
				}
			}
		}

		/**
		 * Show Envato token settings for auto module updates.
		 */
		public function wkpu_updates_show_envato_token_settings() {
			?>
			<form method="post" action="options.php" class="wkwc_envato_token_form">
				<?php
				settings_fields( 'wkwc_envato_token_settings_group' );
				do_settings_sections( 'wkwc_envato_token_settings' );
				submit_button();
				$connection_status = $this->authorize_token_permissions();

				$msg = ( 'success' === $connection_status ) ? esc_html__( 'Connected.', 'wkpu_updates' ) : esc_html__( 'Invalid token.', 'wkpu_updates' );
				$msg = ( 'missing-permissions' === $connection_status ) ? esc_html__( 'Token have some required permission missing.', 'wkpu_updates' ) : $msg;
				$msg = ( 'too-many-permissions' === $connection_status ) ? esc_html__( 'Token have additional permissions.', 'wkpu_updates' ) : $msg;

				$error_transient = get_site_transient( WKPU_UPDATE_OPTION_NAME . '_error_information' );
				if ( ! empty( $error_transient['title'] ) ) {
					$msg .= ' ' . $error_transient['title'] . ': ' . $error_transient['message'];
				}

				$pu_page = filter_input( INPUT_GET, 'page', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
				?>
				<div class="wkwc-evanto-test-wrap">
					<a href="<?php echo esc_url( admin_url( 'admin.php?page=' . $pu_page . '&wkpu_auth_check=token' ) ); ?>" class="button button-secondary wkwc-envato-auth-check-button"><?php esc_html_e( 'Test API Connection', 'wkpu_updates' ); ?></a>
					<span id="wkwc_envato_spinner" class="spinner"></span>
					<span class="wkwc-envato-result <?php echo ( 'success' === $connection_status ) ? esc_attr( 'wkwc-env-success' ) : esc_attr( 'wkwc-env-error' ); ?>" id="wkwc_envato_test_result"><?php echo esc_html( $msg ); ?></span>
				</div>
			</form>
			<?php
		}

		/**
		 * Callback function for the "Envato Token Settings" section.
		 */
		public function wkpu_updates_init_envato_token_settings() {
			register_setting( 'wkwc_envato_token_settings_group', 'wkwc_envato_user_token', array( $this, 'wkwc_validate_envato_token' ) );

			// Register a new section in the "Envato Token" page.
			add_settings_section(
				'wkwc_envato_token_settings_section',
				esc_html__( 'Envato Token Settings', 'wkpu_updates' ),
				array( $this, 'wkwc_envato_token_settings_section_cb' ),
				'wkwc_envato_token_settings'
			);

			// Register a new field in the "Envato Settings Section" section, inside the "Envato Settings" page.
			add_settings_field(
				'wkwc_envato_user_token_field',
				esc_html__( 'Envato Token', 'wkpu_updates' ),
				array( $this, 'wkwc_envato_user_token_field_cb' ),
				'wkwc_envato_token_settings',
				'wkwc_envato_token_settings_section'
			);

			$auth_check = filter_input( INPUT_GET, 'wkpu_auth_check', FILTER_SANITIZE_FULL_SPECIAL_CHARS );

			if ( 'token' === $auth_check ) {
				$db_token = get_option( 'wkwc_envato_user_token', false );

				$this->authorize_token_permissions( $db_token );

				$pu_page = filter_input( INPUT_GET, 'page', FILTER_SANITIZE_FULL_SPECIAL_CHARS );

				wp_safe_redirect( admin_url( 'admin.php?page=' . $pu_page ) );
				exit( 1 );
			}
		}

		/**
		 * Validates the provided Envato token.
		 *
		 * This function checks if the provided token is valid or not.
		 * It can be used to validate the token before using it for any API calls.
		 *
		 * @param string $token The Envato token to be validated.
		 *
		 * @return string|WP_Error
		 */
		public function wkwc_validate_envato_token( $token ) {
			if ( ! empty( $token ) && strlen( $token ) > 5 ) {
				$db_token = get_option( 'wkwc_envato_user_token', false );

				if ( empty( $db_token ) || $db_token !== $token ) {
					$this->authorize_token_permissions( $token );
				}
				return $token;
			} else {
				add_settings_error( '_wkwc_envato_invalid_token', 'token-error', esc_html__( 'Invalid or empty token.', 'wkpu_updates' ), 'error' );
			}
		}

		/**
		 * Adding token settings section.
		 */
		public function wkwc_envato_token_settings_section_cb() {
			?>
			<p><?php esc_html_e( 'Please add the Envato token below to update the Envato plugins directly from the Plugin\'s page. It saves the effort of manually downloading the plugin from CodeCanyon and uploading it to the WordPress Plugin update page.', 'wkpu_updates' ); ?></p>
			<p><?php esc_html_e( 'Follow the steps below to create a new token.', 'wkpu_updates' ); ?></p>

			<ol>
				<li><?php printf( /* translators: %s: Envato token link. */ esc_html__( 'Generate an Envato API Personal Token by %s.', 'wkpu_updates' ), '<a href="' . esc_url( $this->get_generate_token_url() ) . '" target="_blank">' . esc_html__( 'clicking this link', 'wkpu_updates' ) . '</a>' ); ?></li>
				<li><?php esc_html_e( 'Name the token eg “WooCommerce Store”.', 'wkpu_updates' ); ?></li>
				<li><?php esc_html_e( 'Ensure the following permissions are enabled:', 'wkpu_updates' ); ?>
					<ul>
						<li><?php esc_html_e( 'View and search Envato sites', 'wkpu_updates' ); ?></li>
						<li><?php esc_html_e( 'Download your purchased items', 'wkpu_updates' ); ?></li>
					</ul>
				</li>
				<li><?php esc_html_e( 'Copy the token into the box below.', 'wkpu_updates' ); ?></li>
				<li><?php esc_html_e( 'Click the "Save Changes" button.', 'wkpu_updates' ); ?></li>
			</ol>
			<?php
		}

		/**
		 * Adding token field.
		 */
		public function wkwc_envato_user_token_field_cb() {
			$setting = get_option( 'wkwc_envato_user_token', false );
			?>
			<input size="40" type="text" id="wkwc_envato_user_token_input" name="wkwc_envato_user_token" value="<?php echo empty( $setting ) ? '' : esc_attr( $setting ); ?>">
			<?php
		}

		/**
		 * Return the URL a user needs to click to generate a personal token.
		 *
		 * @single 2.0.1
		 *
		 * @return string The full URL to request a token.
		 */
		public function get_generate_token_url() {
			return 'https://build.envato.com/create-token/?' . implode(
				'&',
				array_map(
					function ( $val ) {
						return $val . '=t';
					},
					array_keys( $this->get_required_permissions() )
				)
			);
		}

		/**
		 * Disables requests to the wp.org repository for our Webkul's Pro plugins.
		 *
		 * @param array  $request An array of HTTP request arguments.
		 * @param string $url The request URL.
		 *
		 * @return array
		 */
		public function wkpu_updates_api_requests( $request, $url ) {
			// Plugin update request.
			if ( false !== strpos( $url, '//api.wordpress.org/plugins/update-check/1.1/' ) ) {
				/**
				 * Exclude plugin slugs that should never ping the WordPress API.
				 * We don't need the extra http requests for our plugins.
				 */
				$wkpu_data = $this->wkpu_get_plugin_data();

				// Decode JSON so we can manipulate the array.
				$data = json_decode( $request['body']['plugins'] );

				foreach ( $wkpu_data as $item_id => $plug_data ) {
					if ( is_numeric( $item_id ) ) {
						$slug = empty( $plug_data['basename'] ) ? '' : dirname( trim( $plug_data['basename'] ) );
						if ( ! empty( $slug ) ) {
							unset( $data->plugins->$slug );
						}
					}
				}

				// Encode back into JSON and update the response.
				$request['body']['plugins'] = wp_json_encode( $data );
			}

			return $request;
		}

		/**
		 * Inject update data for webkul plugins.
		 *
		 * @param object $transient The pre-saved value of the `update_plugins` site transient.
		 *
		 * @return object
		 */
		public function wkpu_update_transient_plugins( $transient ) {
			$wkpu_data     = $this->wkpu_get_plugin_data();
			$installed_ids = array();

			foreach ( $wkpu_data as $item_id => $plug_data ) {
				if ( is_numeric( $item_id ) && ! empty( $item_id ) ) {
					$basename = empty( $plug_data['basename'] ) ? '' : trim( $plug_data['basename'] );

					$installed_ids[ $item_id ] = false;

					if ( ! empty( $plug_data['installed'] ) && ! empty( $plug_data['version'] ) && version_compare( $plug_data['version'], $plug_data['installed'], '>' ) ) {
						$installed_ids[ $item_id ] = true;

						// Add plugin to transient.
						$transient->response[ $basename ] = (object) array(
							'slug'        => dirname( $basename ),
							'plugin'      => $basename,
							'new_version' => $plug_data['version'],
							'tested'      => $plug_data['tested'],
							'url'         => $plug_data['url'],
							'package'     => $this->deferred_download( $item_id, $basename ),
							'icons'       => array( '2x' => $plug_data['thumbnail_url'] ),
						);
					}
				}
			}

			$purchased_plugins = $this->wkpu_get_plugin_data( '', '_purchased_plugins' );
			$purchased_plugins = empty( $purchased_plugins ) ? array() : $purchased_plugins;

			foreach ( $purchased_plugins as $purchased ) {
				$item_id         = empty( $purchased['id'] ) ? '' : $purchased['id'];
				$author_username = empty( $purchased['author'] ) ? '' : $purchased['author'];

				if ( ! empty( $item_id ) && ! empty( $author_username ) && 'webkul' === strtolower( $author_username ) && ( ! array_key_exists( $item_id, $installed_ids ) || empty( $installed_ids[ $item_id ] ) ) ) {
					$basename = ( array_key_exists( $item_id, $wkpu_data ) && ! empty( $wkpu_data[ $item_id ]['basename'] ) ) ? trim( $wkpu_data[ $item_id ]['basename'] ) : '';

					if ( ! empty( $basename ) ) {
						$plugin_data = get_plugin_data( WP_PLUGIN_DIR . '/' . $basename );

						if ( ! empty( $plugin_data['Version'] ) && ! empty( $purchased['version'] ) && version_compare( $purchased['version'], $plugin_data['Version'], '>' ) ) {
							$installed_ids[ $item_id ] = true;

							// Add plugin to transient.
							$transient->response[ $basename ] = (object) array(
								'slug'        => dirname( $basename ),
								'plugin'      => $basename,
								'new_version' => $purchased['version'],
								'tested'      => $plug_data['tested'],
								'url'         => $purchased['url'],
								'package'     => $this->deferred_download( $item_id, $basename ),
								'icons'       => array( '2x' => $purchased['thumbnail_url'] ),
							);
						}
					}
				}
			}

			return $transient;
		}

		/**
		 * Deferred item download URL to Envato URL for Webkul's pro plugins.
		 *
		 * @param int    $item_id The item ID.
		 * @param string $basename Plugins basename.
		 *
		 * @return string.
		 */
		public function deferred_download( $item_id, $basename ) {
			if ( empty( $item_id ) ) {
				return '';
			}

			if ( ! $this->envato_connected() ) {
				return false;
			}

			$purchased_plugins  = $this->wkpu_get_plugin_data( '', '_purchased_plugins' );
			$purchased_item_ids = wp_list_pluck( $purchased_plugins, 'id' );

			if ( ! in_array( $item_id, $purchased_item_ids, true ) ) {
				return false;
			}

			$download_url = is_plugin_active_for_network( $basename ) ? network_admin_url( 'admin.php?page=wkwc-addons' ) : admin_url( 'admin.php?page=wkwc-addons' );

			$args = array(
				'deferred_download' => true,
				'item_id'           => $item_id,
			);
			return add_query_arg( $args, esc_url( $download_url ) );
		}

		/**
		 * Inject API data for Webkul's premium plugins.
		 *
		 * @param bool   $response Always false.
		 * @param string $action The API action being performed.
		 * @param object $args Plugin arguments.
		 *
		 * @hooked 'plugins_api' filter hook.
		 *
		 * @return bool|object $response The plugin info or false.
		 */
		public function wkpu_update_add_view_details_popup_info( $response, $action, $args ) {
			// Process Webkul plugin updates.

			if ( 'plugin_information' === $action && isset( $args->slug ) ) {
				$wkpu_data = $this->wkpu_get_plugin_data( '', '', $args->slug );

				$info_data      = array();
				$plugin_item_id = 0;

				foreach ( $wkpu_data as $item_id => $plug_data ) {
					if ( is_numeric( $item_id ) && dirname( $plug_data['basename'] ) === $args->slug && ! empty( $plug_data['plugin_name'] ) ) {
						$plugin_item_id = $item_id;
						$basename       = empty( $plug_data['basename'] ) ? '' : trim( $plug_data['basename'] );

						$info_data['slug']         = $args->slug;
						$info_data['plugin']       = $basename;
						$info_data['plugin_name']  = $plug_data['plugin_name'];
						$info_data['name']         = $plug_data['plugin_name'];
						$info_data['version']      = $plug_data['version'];
						$info_data['author']       = $plug_data['author'];
						$info_data['homepage']     = $plug_data['url'];
						$info_data['downloaded']   = $plug_data['number_of_sales'];
						$info_data['last_updated'] = $plug_data['updated_at'];
						$info_data['rating']       = ! empty( $plug_data['rating'] ) && $plug_data['rating'] > 0 ? $plug_data['rating'] / 5 * 100 : 0;
						$info_data['num_ratings']  = empty( $plug_data['rating_count'] ) ? 0 : $plug_data['rating_count'];

						$info_data['description']   = empty( $plug_data['description'] ) ? '' : $plug_data['description'];
						$info_data['landscape_url'] = empty( $plug_data['landscape_url'] ) ? '' : $plug_data['landscape_url'];
						$info_data['changelog']     = empty( $plug_data['changelog'] ) ? '' : $plug_data['changelog'];

						break;
					}
				}

				if ( empty( $info_data ) ) {
					$purchased_plugins = $this->wkpu_get_plugin_data( '', '_purchased_plugins', $args->slug );
					$purchased_plugins = empty( $purchased_plugins ) ? array() : $purchased_plugins;

					foreach ( $purchased_plugins as $purchased ) {
						$item_id         = empty( $purchased['id'] ) ? '' : $purchased['id'];
						$author_username = empty( $purchased['author'] ) ? '' : $purchased['author'];

						if ( ! empty( $item_id ) && ! empty( $author_username ) && 'webkul' === strtolower( $author_username ) ) {
							$basename = ( array_key_exists( $item_id, $wkpu_data ) && ! empty( $wkpu_data[ $item_id ]['basename'] ) ) ? trim( $wkpu_data[ $item_id ]['basename'] ) : '';

							if ( ! empty( $basename ) ) {
								if ( is_numeric( $item_id ) && dirname( $basename ) === $args->slug && ! empty( $purchased['plugin_name'] ) ) {
									$plugin_item_id = $item_id;

									$info_data['slug']         = $args->slug;
									$info_data['plugin']       = $basename;
									$info_data['plugin_name']  = $purchased['plugin_name'];
									$info_data['name']         = $purchased['plugin_name'];
									$info_data['version']      = $purchased['version'];
									$info_data['author']       = $author_username;
									$info_data['homepage']     = $purchased['url'];
									$info_data['downloaded']   = $purchased['number_of_sales'];
									$info_data['last_updated'] = $purchased['updated_at'];
									$info_data['rating']       = ! empty( $purchased['rating'] ) && $purchased['rating'] > 0 ? $purchased['rating'] / 5 * 100 : 0;
									$info_data['num_ratings']  = empty( $purchased['rating_count'] ) ? 0 : $purchased['rating_count'];

									$info_data['description']   = empty( $purchased['description'] ) ? '' : $purchased['description'];
									$info_data['landscape_url'] = empty( $purchased['landscape_url'] ) ? '' : $purchased['landscape_url'];
									$info_data['changelog']     = $this->parse_description_for_changelog( $purchased, $purchased['url'] );

									break;
								}
							}
						}
					}
				}

				if ( ! empty( $info_data ) && ! empty( $plugin_item_id ) ) {
					$response = new stdClass();

					foreach ( $info_data as $res_key => $info ) {
						if ( in_array( $res_key, array( 'description', 'landscape_url', 'changelog' ), true ) ) {
							continue;
						}
						$response->{$res_key} = $info;
					}

					$plugin_basename = empty( $info_data['plugin'] ) ? '' : $info_data['plugin'];

					$response->tested        = get_bloginfo( 'version' );
					$response->download_link = $this->deferred_download( $plugin_item_id, $plugin_basename );

					$description = empty( $info_data['description'] ) ? '' : $info_data['description'];

					if ( ! empty( $plugin_basename ) ) {
						$fields = array(
							'RequiresWP' => 'Requires at least',
							'Blog URI'   => 'Blog URI',
						);

						$plugin_data = get_file_data( WP_PLUGIN_DIR . '/' . $plugin_basename, $fields, 'plugin' );

						$response->requires = empty( $plugin_data['RequiresWP'] ) ? get_bloginfo( 'version' ) : $plugin_data['RequiresWP'];

						if ( ! empty( $plugin_data['Blog URI'] ) ) {
							$description .= ' <br/><br/><a href="' . esc_url( $plugin_data['Blog URI'] ) . '">' . esc_html__( 'See the complete blog', 'wkpu_updates' ) . '</a>';
						}
					}

					$changelog = empty( $info_data['changelog'] ) ? '' : $info_data['changelog'];

					if ( ! empty( $changelog ) ) {
						$changelog = nl2br( $changelog );

						$first_line = explode( '<br />', $changelog )[0];
						$changelog  = str_replace( $first_line, '<strong>' . $first_line . '</strong><br/>', $changelog );
					}

					$response->sections = array(
						'description' => $description,
						'changelog'   => wp_kses_post( $changelog ),
					);

					$response->banners['low'] = $info_data['landscape_url'];
				}
			}

			return $response;
		}

		/**
		 * Parse API response data to extract changelog from CodeCanyon description.
		 *
		 * @param array  $api_data API data array.
		 * @param string $plugin_url Plugin Item URL.
		 *
		 * @return string
		 */
		public function parse_description_for_changelog( $api_data, $plugin_url ) {
			$description = empty( $api_data['description'] ) ? '' : $api_data['description'];
			$elem_id     = 'item-description__changelog';

			// Use regex to find the position of the element with the specified ID.
			$pattern = '/<[^>]*id=["\']' . preg_quote( $elem_id, '/' ) . '["\'][^>]*>/i';
			preg_match( $pattern, $description, $matches, PREG_OFFSET_CAPTURE );

			if ( empty( $matches ) ) {
				$elem_id = 'item-description__change-log';
				$pattern = '/<[^>]*id=["\']' . preg_quote( $elem_id, '/' ) . '["\'][^>]*>/i';
				preg_match( $pattern, $description, $matches, PREG_OFFSET_CAPTURE );
			}

			$url = empty( $api_data['url'] ) ? $plugin_url : $api_data['url'];

			$change_log_link = '<br/><br/><a href="' . $url . '#' . $elem_id . '">' . esc_html__( 'See changelog for all versions', 'wkpu_updates' ) . '</a>';

			if ( empty( $matches ) ) {
				return $change_log_link; // Element with the specified ID not found. Returning item URL.
			}

			// Get the position of the found element.
			$position = $matches[0][1];

			// Extract the substring from that position onwards.
			$substring = substr( $description, $position );

			// Find all <pre> tags in the substring.
			preg_match_all( '/<pre>(.*?)<\/pre>/is', $substring, $preMatches );

			// Check if there are at least two <pre> tags.
			if ( count( $preMatches[1] ) < 2 ) {
				return $change_log_link; // Less than two <pre> tags found.
			}

			// Return the content of the second <pre> tag along with the item URL.
			return trim( $preMatches[1][1] ) . $change_log_link;
		}

		/**
		 * Adding View Details link and removing 'Visit plugin Site' link.
		 *
		 * @param array  $links Meta links.
		 * @param string $file File name.
		 */
		public function wkpu_add_view_details_row_meta( $links, $file ) {
			$wkpu_data = $this->wkpu_get_plugin_data( '', '', $file );

			foreach ( $wkpu_data as $item_id => $plug_data ) {
				if ( is_numeric( $item_id ) ) {
					$basename = empty( $plug_data['basename'] ) ? '' : trim( $plug_data['basename'] );

					if ( $file === $basename && ! empty( $plug_data['url'] ) ) {
						$view_details_exist = false;
						foreach ( $links as $key => $link ) {
							preg_match( '~<a[^>]+href=["\']([^"\']+)["\'][^>]*>~', $link, $matches );

							if ( isset( $matches[1] ) && $plug_data['url'] === $matches[1] ) {
								unset( $links[ $key ] );
							}

							if ( isset( $matches[1] ) && stripos( $matches[1], 'plugin-install.php?tab=plugin-information' ) > 0 ) {
								$view_details_exist = true;
							}
						}

						if ( ! $view_details_exist ) {
							$install_path = 'plugin-install.php?tab=plugin-information&plugin=' . dirname( $basename ) . '&TB_iframe=true&width=772&height=650';
							$install_url  = is_multisite() ? network_admin_url( $install_path ) : admin_url( $install_path );
							$links[]      = '<a href="' . esc_url( $install_url ) . '" class="thickbox open-plugin-details-modal" data-title="' . esc_attr( $plug_data['plugin_name'] ) . '" aria-label="' . esc_attr__( 'More information about ', 'wkpu_updates' ) . esc_attr( $plug_data['plugin_name'] ) . '">' . esc_html__( 'View Details', 'wkpu_updates' ) . '</a>';
						}

						break;
					}
				}
			}

			return $links;
		}

		/**
		 * Get plugin's collected data for updates.
		 *
		 * @param string $from Get our plugin's data from, e.g. 'transient' or 'db' or 'both'.
		 * @param string $key Option or transient key to retrieve.
		 * @param string $single_item Single plugin basename to retrieve.
		 *
		 * @return array
		 */
		public function wkpu_get_plugin_data( $from = '', $key = '', $single_item = '' ) {
			$wkpu_data    = array();
			$wkpu_db_data = array();

			$from = empty( $from ) ? 'both' : $from;
			$key  = empty( $key ) ? '_wk_plugins_data' : $key;

			if ( 'db' !== $from ) {
				$wkpu_data = get_site_transient( WKPU_UPDATE_OPTION_NAME . $key );

				if ( 'transient' === $from || ! empty( $wkpu_data ) ) {
					$wkpu_data = empty( $wkpu_data ) ? array() : $wkpu_data;

					if ( ! empty( $single_item ) ) {
						$single_data = array();

						if ( 1 === count( explode( '/', $single_item ) ) ) {
							$single_data = wp_list_filter( $wkpu_data, array( 'basename' => $single_item . '/' . $single_item . '.php' ) );
						} else {
							$single_data = wp_list_filter( $wkpu_data, array( 'basename' => $single_item ) );
						}

						if ( ! empty( $single_data ) ) {
							return $single_data;
						}
					}

					return $wkpu_data;
				}
			}

			$wkpu_db_data = get_option( WKPU_UPDATE_OPTION_NAME . $key, array() );
			$wkpu_db_data = empty( $wkpu_db_data ) ? array() : $wkpu_db_data;

			if ( ! empty( $single_item ) ) {
				if ( 1 === count( explode( '/', $single_item ) ) ) {
					return wp_list_filter( $wkpu_db_data, array( 'basename' => $single_item . '/' . $single_item . '.php' ) );
				}
				return wp_list_filter( $wkpu_db_data, array( 'basename' => $single_item ) );
			}

			return $wkpu_db_data;
		}

		/**
		 * Show notices on progress.
		 */
		public function wkpu_updates_maybe_show_notices() {
			if ( defined( 'WKWC_DEV' ) && WKWC_DEV ) {
				$totals = get_site_transient( WKPU_UPDATE_OPTION_NAME . '_totals' );

				if ( ! empty( $totals ) ) {
					$pushed_count = $totals['pushed_count'] ?? 0;
					$completed    = $totals['completed'] ?? array();

					if ( ! empty( $pushed_count ) ) {
						$message = wp_sprintf( /* translators: %1$d: Completed count, %2$d: Pushed count. */ esc_html__( '%1$d out of %2$d plugins meta has been updated.', 'wkpu_updates' ), count( $completed ), $pushed_count );
						wp_admin_notice( $message, array( 'type' => 'success' ) );
					}
				}
			}
		}

		/**
		 * Show current plugin version and last working date and time on front end.
		 *
		 * @hooked wp_footer Action hook.
		 *
		 * @return void
		 */
		public function wkpu_updates_front_footer_info() {
			$show_info = filter_input( INPUT_GET, 'wkmodule_info', FILTER_SANITIZE_NUMBER_INT );
			$show_info = empty( $show_info ) ? 0 : intval( $show_info );

			if ( 200 === $show_info ) {
				$wkpu_data = $this->wkpu_get_plugin_data();

				foreach ( $wkpu_data as $item_id => $plugin_info ) {
					$basename = empty( $plugin_info['basename'] ) ? '' : $plugin_info['basename'];
					if ( empty( $basename ) ) {
						continue;
					}

					$code   = empty( $plugin_info['plugin_name'] ) ? '' : get_option( 'wp_pcc_' . str_replace( ' ', '_', strtolower( $plugin_info['plugin_name'] ) ), false );
					$p_data = empty( $code ) ? array() : maybe_unserialize( $code );
					$output = 'item_id-' . $item_id . '=pcc-';

					if ( ! empty( $p_data['supported_until'] ) ) {
						$p_code = empty( $p_data['purchase_code'] ) ? '' : $p_data['purchase_code'];

						$output .= empty( $p_code ) ? '' : substr( $p_code, 0, 4 ) . '-' . substr( $p_code, -4 );
						$output .= empty( $p_data['supported_until'] ) ? '' : '-' . str_replace( '-', '', substr( $p_data['supported_until'], 0, 10 ) );
					}

					$plugin_data = get_plugin_data( WP_PLUGIN_DIR . '/' . $basename, array( 'WKLWDT' => 'WKLWDT' ) );
					$last_worked = empty( $plugin_data['WKLWDT'] ) ? '' : 'data-lwdt=' . $plugin_data['WKLWDT'];

					$installed_version = empty( $plugin_info['installed'] ) ? '' : $plugin_info['installed'];
					$version_output    = dirname( $basename ) . '=' . $installed_version;
					$active_output     = 'is_active=';
					$active_output    .= is_plugin_active( $basename ) ? '1' : '0';
					?>
					<input type="hidden" <?php echo esc_attr( $active_output . ' ' . $output . ' ' . $version_output . ' ' . $last_worked ); ?>>
					<?php
				}
				?>
				<input type="hidden" data-lwdt="202411051630" wk-plugin-updates="<?php echo esc_attr( WKPU_UPDATE_VERSION ); ?>">
				<?php
			}
		}
	}
}
