<?php
/**
 * All functionality to update plugin meta in background every six hours.
 *
 * @package WK Plugin Updates
 */

defined( 'ABSPATH' ) || exit(); // Exit if access directly.

if ( ! class_exists( 'WKPU_Updates_Background_Process', false ) ) {
	require_once __DIR__ . '/class-wkpu-updates-background-process.php';
}

/**
 * Class that extends WKPU_UPDATES_Background_Plugins to process plugins meta data updates in background every six hours.
 */
class WKPU_UPDATES_Background_Plugins extends WKPU_Updates_Background_Process {
	/**
	 * Update object.
	 *
	 * @var mixed
	 */
	protected $update_obj;

	/**
	 * Initiate new background process.
	 */
	public function __construct() {
		// Uses unique prefix per blog so each blog has separate queue.
		$this->prefix = 'wkwc_pu_updates_' . get_current_blog_id();
		$this->action = 'wkwc_pu_updates';

		parent::__construct();
	}

	/**
	 * Is a job running?
	 *
	 * @return boolean
	 */
	public function is_running() {
		if ( $this->is_queue_empty() ) {
			return false;
		}
		return true;
	}

	/**
	 * Code to execute for each item in the queue
	 *
	 * @param mixed $item Queue item to iterate over.
	 * @return bool
	 */
	protected function task( $item ) {
		update_option( $this->identifier, time() );

		$item_id          = $item['item_id'] ?? false;
		$this->update_obj = empty( $this->update_obj ) ? ( $item['update_obj'] ?? false ) : $this->update_obj;

		WKPU_Updates::log( 'Background update task started for item id: ' . $item_id );

		if ( empty( $item_id ) || ! $this->update_obj instanceof \WKPU_Updates ) {
			return false;
		}

		$plugin_data = $item['plugin_data'] ?? array();
		$wk_plugins  = $this->update_obj->wkpu_get_plugin_data( 'transient' );

		$wk_plugins[ $item_id ] = $this->update_obj->update_plugin_meta( $item_id, $plugin_data );

		set_site_transient( WKPU_UPDATE_OPTION_NAME . '_wk_plugins_data', $wk_plugins, 6 * HOUR_IN_SECONDS );

		$totals       = get_site_transient( WKPU_UPDATE_OPTION_NAME . '_totals' );
		$pushed_count = $totals['pushed_count'] ?? 0;
		$completed    = $totals['completed'] ?? array();

		if ( ! in_array( $item_id, $completed, true ) ) {
			$completed[] = $item_id;
		}

		$completed_count = count( $completed );

		$totals['completed'] = $completed;

		set_site_transient( WKPU_UPDATE_OPTION_NAME . '_totals', $totals, 10 * MINUTE_IN_SECONDS );

		$message = wp_sprintf( /* translators: %1$d: Completed count, %2$d: Pushed count. */ esc_html__( '%1$d out of %2$d plugins meta has been updated.', 'wkpu_updates' ), $completed_count, $pushed_count );

		WKPU_Updates::log( $message );

		return false;
	}

	/**
	 * Checks if a background process needs recovery.
	 *
	 * This function compares the last execution time of a given task with the current time.
	 * If the task hasn't been executed in the last 10 minutes (600 seconds), it indicates that the process might be stuck.
	 * In such cases, the function returns true, indicating that recovery is needed.
	 *
	 * @param string $task_name The name of the task for which recovery needs to be checked.
	 *
	 * @return bool Returns true if recovery is needed, false otherwise.
	 */
	public static function wkpu_updates_background_process_needs_recovery( $task_name ) {
		$last_execution_time = get_option( $task_name );
		$current_time        = time();

		// Check if the process is running and task function hasn't executed in the last 10 minutes.
		if ( $last_execution_time && ( $current_time - $last_execution_time ) > 600 ) {
			// Kill and restart the process.
			return true;
		}
		return false;
	}

	/**
	 * This runs once the job has completed all items on the queue.
	 *
	 * @return void
	 */
	protected function complete() {
		parent::complete();

		$totals       = get_site_transient( WKPU_UPDATE_OPTION_NAME . '_totals' );
		$pushed_count = $totals['pushed_count'] ?? 0;

		delete_site_transient( WKPU_UPDATE_OPTION_NAME . '_totals' );

		$wk_plugins = $this->update_obj->wkpu_get_plugin_data( 'transient' );

		update_option( WKPU_UPDATE_OPTION_NAME . '_wk_plugins_data', $wk_plugins );

		WKPU_Updates::log( 'Background Process Completed. All ' . $pushed_count . ' items meta updated. Updated plugins ids are: ' . print_r( $totals['completed'], true ) );
	}
}
